#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Windows preprocessor plugins."""

import unittest

from dfvfs.helpers import fake_file_system_builder
from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.path import factory as path_spec_factory
from dfwinreg import fake as dfwinreg_fake
from dfwinreg import regf as dfwinreg_regf
from dfwinreg import registry as dfwinreg_registry
from dfwinreg import registry_searcher

from plaso.containers import artifacts
from plaso.preprocessors import manager
from plaso.preprocessors import mediator
from plaso.preprocessors import windows

from tests.preprocessors import test_lib


class WindowsArtifactPreprocessorPluginTestCase(
    test_lib.ArtifactPreprocessorPluginTestCase):
  """Windows artifact preprocessor plugin test case."""

  def _GetWinRegistryFromFileEntry(self, file_entry):
    """Retrieves a Windows Registry from a file entry.

    Args:
      file_entry (dfvfs.FileEntry): file entry that references a test file.

    Returns:
      dfwinreg.WinRegistry: Windows Registry or None.
    """
    file_object = file_entry.GetFileObject()
    if not file_object:
      return None

    registry_file = dfwinreg_regf.REGFWinRegistryFile(ascii_codepage='cp1252')
    registry_file.Open(file_object)

    win_registry = dfwinreg_registry.WinRegistry()
    key_path_prefix = win_registry.GetRegistryFileMapping(registry_file)
    win_registry.MapFile(key_path_prefix, registry_file)

    return win_registry

  def _RunPreprocessorPluginOnWindowsRegistryValue(
      self, file_system, mount_point, storage_writer, plugin):
    """Runs a preprocessor plugin on a Windows Registry value.

    Args:
      file_system (dfvfs.FileSystem): file system to be preprocessed.
      mount_point (dfvfs.PathSpec): mount point path specification that refers
          to the base location of the file system.
      storage_writer (StorageWriter): storage writer.
      plugin (ArtifactPreprocessorPlugin): preprocessor plugin.

    Return:
      PreprocessMediator: preprocess mediator.
    """
    artifact_definition = self._artifacts_registry.GetDefinitionByName(
        plugin.ARTIFACT_DEFINITION_NAME)
    self.assertIsNotNone(artifact_definition)

    environment_variable = artifacts.EnvironmentVariableArtifact(
        case_sensitive=False, name='SystemRoot', value='C:\\Windows')

    registry_file_reader = manager.FileSystemWinRegistryFileReader(
        file_system, mount_point, environment_variables=[environment_variable])
    win_registry = dfwinreg_registry.WinRegistry(
        registry_file_reader=registry_file_reader)

    test_mediator = mediator.PreprocessMediator(storage_writer)

    searcher = registry_searcher.WinRegistrySearcher(win_registry)

    plugin.Collect(test_mediator, artifact_definition, searcher)

    return test_mediator

  def _RunPreprocessorPluginOnWindowsRegistryValueSoftware(
      self, storage_writer, plugin):
    """Runs a preprocessor plugin on a Windows Registry value in SOFTWARE.

    Args:
      storage_writer (StorageWriter): storage writer.
      plugin (ArtifactPreprocessorPlugin): preprocessor plugin.

    Return:
      PreprocessMediator: preprocess mediator.
    """
    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    test_file_path = self._GetTestFilePath(['SOFTWARE'])
    file_system_builder.AddFileReadData(
        '/Windows/System32/config/SOFTWARE', test_file_path)

    mount_point = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_FAKE, location='/')

    return self._RunPreprocessorPluginOnWindowsRegistryValue(
        file_system_builder.file_system, mount_point, storage_writer, plugin)

  def _RunPreprocessorPluginOnWindowsRegistryValueSystem(
      self, storage_writer, plugin):
    """Runs a preprocessor plugin on a Windows Registry value in SYSTEM.

    Args:
      storage_writer (StorageWriter): storage writer.
      plugin (ArtifactPreprocessorPlugin): preprocessor plugin.

    Return:
      PreprocessMediator: preprocess mediator.
    """
    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    test_file_path = self._GetTestFilePath(['SYSTEM'])
    file_system_builder.AddFileReadData(
        '/Windows/System32/config/SYSTEM', test_file_path)

    mount_point = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_FAKE, location='/')

    return self._RunPreprocessorPluginOnWindowsRegistryValue(
        file_system_builder.file_system, mount_point, storage_writer, plugin)


class WindowsAllUsersAppDataKnowledgeBasePluginTest(
    test_lib.ArtifactPreprocessorPluginTestCase):
  """Tests for the allusersdata knowledge base value plugin."""

  # pylint: disable=protected-access

  def testCollect(self):
    """Tests the Collect function."""
    plugin = windows.WindowsAllUsersAppDataKnowledgeBasePlugin()

    storage_writer = self._CreateTestStorageWriter()
    test_mediator = mediator.PreprocessMediator(storage_writer)

    plugin.Collect(test_mediator)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'preprocessing_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_artifacts = storage_writer.GetNumberOfAttributeContainers(
        'environment_variable')
    self.assertEqual(number_of_artifacts, 0)

  def testCollectWithAllUsersProfile(self):
    """Tests the Collect function with the %AllUsersProfile% variable."""
    plugin = windows.WindowsAllUsersAppDataKnowledgeBasePlugin()

    storage_writer = self._CreateTestStorageWriter()
    test_mediator = mediator.PreprocessMediator(storage_writer)

    environment_variable = artifacts.EnvironmentVariableArtifact(
        case_sensitive=False, name='allusersprofile',
        value='C:\\Documents and Settings\\All Users')

    test_mediator._environment_variables['ALLUSERSPROFILE'] = (
        environment_variable)

    plugin.Collect(test_mediator)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'preprocessing_warning')
    self.assertEqual(number_of_warnings, 0)

    # The %AllUsersAppData% environment variable is derived from
    # the %AllUsersProfile% environment variable.
    number_of_artifacts = storage_writer.GetNumberOfAttributeContainers(
        'environment_variable')
    self.assertEqual(number_of_artifacts, 1)

    environment_variable = storage_writer.GetAttributeContainerByIndex(
        'environment_variable', 0)
    self.assertIsNotNone(environment_variable)
    self.assertEqual(environment_variable.name, 'allusersappdata')
    self.assertEqual(
        environment_variable.value,
        'C:\\Documents and Settings\\All Users\\Application Data')

  def testCollectWithProgramData(self):
    """Tests the Collect function with the %ProgramData% variable."""
    plugin = windows.WindowsAllUsersAppDataKnowledgeBasePlugin()

    storage_writer = self._CreateTestStorageWriter()
    test_mediator = mediator.PreprocessMediator(storage_writer)

    environment_variable = artifacts.EnvironmentVariableArtifact(
        case_sensitive=False, name='programdata',
        value='%SystemDrive%\\ProgramData')

    test_mediator._environment_variables['PROGRAMDATA'] = environment_variable

    plugin.Collect(test_mediator)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'preprocessing_warning')
    self.assertEqual(number_of_warnings, 0)

    # The %AllUsersAppData% environment variable is derived from
    # the %ProgramData% environment variable.
    number_of_artifacts = storage_writer.GetNumberOfAttributeContainers(
        'environment_variable')
    self.assertEqual(number_of_artifacts, 1)

    environment_variable = storage_writer.GetAttributeContainerByIndex(
        'environment_variable', 0)
    self.assertIsNotNone(environment_variable)
    self.assertEqual(environment_variable.name, 'allusersappdata')
    self.assertEqual(environment_variable.value, '%SystemDrive%\\ProgramData')


class WindowsAllUsersProfileEnvironmentVariablePluginTest(
    WindowsArtifactPreprocessorPluginTestCase):
  """Tests for the %AllUsersProfile% environment variable plugin."""

  def testParseValueData(self):
    """Tests the _ParseValueData function."""
    test_file_path = self._GetTestFilePath(['SOFTWARE'])
    self._SkipIfPathNotExists(test_file_path)

    storage_writer = self._CreateTestStorageWriter()

    plugin = windows.WindowsAllUsersProfileEnvironmentVariablePlugin()
    self._RunPreprocessorPluginOnWindowsRegistryValueSoftware(
        storage_writer, plugin)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'preprocessing_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_artifacts = storage_writer.GetNumberOfAttributeContainers(
        'environment_variable')
    self.assertEqual(number_of_artifacts, 0)


class WindowsAllUsersAppProfileKnowledgeBasePluginTest(
    test_lib.ArtifactPreprocessorPluginTestCase):
  """Tests for the allusersprofile knowledge base value plugin."""

  # pylint: disable=protected-access

  def testCollect(self):
    """Tests the Collect function."""
    plugin = windows.WindowsAllUsersAppProfileKnowledgeBasePlugin()

    storage_writer = self._CreateTestStorageWriter()
    test_mediator = mediator.PreprocessMediator(storage_writer)

    plugin.Collect(test_mediator)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'preprocessing_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_artifacts = storage_writer.GetNumberOfAttributeContainers(
        'environment_variable')
    self.assertEqual(number_of_artifacts, 0)

  def testCollectWithAllUsersProfile(self):
    """Tests the Collect function with the %AllUsersProfile% variable."""
    plugin = windows.WindowsAllUsersAppProfileKnowledgeBasePlugin()

    storage_writer = self._CreateTestStorageWriter()
    test_mediator = mediator.PreprocessMediator(storage_writer)

    environment_variable = artifacts.EnvironmentVariableArtifact(
        case_sensitive=False, name='allusersprofile',
        value='C:\\Documents and Settings\\All Users')

    test_mediator._environment_variables['ALLUSERSPROFILE'] = (
        environment_variable)

    plugin.Collect(test_mediator)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'preprocessing_warning')
    self.assertEqual(number_of_warnings, 0)

    # The %AllUsersProfile% environment variable is already set in
    # the knowledge base and should not be created.
    number_of_artifacts = storage_writer.GetNumberOfAttributeContainers(
        'environment_variable')
    self.assertEqual(number_of_artifacts, 0)

  def testCollectWithProgramData(self):
    """Tests the Collect function with the %ProgramData% variable."""
    plugin = windows.WindowsAllUsersAppProfileKnowledgeBasePlugin()

    storage_writer = self._CreateTestStorageWriter()
    test_mediator = mediator.PreprocessMediator(storage_writer)

    environment_variable = artifacts.EnvironmentVariableArtifact(
        case_sensitive=False, name='programdata',
        value='%SystemDrive%\\ProgramData')

    test_mediator._environment_variables['PROGRAMDATA'] = environment_variable

    plugin.Collect(test_mediator)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'preprocessing_warning')
    self.assertEqual(number_of_warnings, 0)

    # The %AllUsersProfile% environment variable is derived from
    # the %ProgramData% environment variable.
    number_of_artifacts = storage_writer.GetNumberOfAttributeContainers(
        'environment_variable')
    self.assertEqual(number_of_artifacts, 1)

    environment_variable = storage_writer.GetAttributeContainerByIndex(
        'environment_variable', 0)
    self.assertIsNotNone(environment_variable)
    self.assertEqual(environment_variable.name, 'allusersprofile')
    self.assertEqual(environment_variable.value, '%SystemDrive%\\ProgramData')


class WindowsAvailableTimeZonesPluginTest(
    WindowsArtifactPreprocessorPluginTestCase):
  """Tests for the Windows available time zones plugin."""

  def testParseKey(self):
    """Tests the _ParseKey function."""
    test_file_path = self._GetTestFilePath(['SOFTWARE'])
    self._SkipIfPathNotExists(test_file_path)

    storage_writer = self._CreateTestStorageWriter()

    plugin = windows.WindowsAvailableTimeZonesPlugin()
    self._RunPreprocessorPluginOnWindowsRegistryValueSoftware(
        storage_writer, plugin)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'preprocessing_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_artifacts = storage_writer.GetNumberOfAttributeContainers(
        'time_zone')
    self.assertEqual(number_of_artifacts, 101)

    available_time_zone = storage_writer.GetAttributeContainerByIndex(
       'time_zone', 7)

    self.assertEqual(available_time_zone.name, 'AUS Central Standard Time')


class WindowsCodePagePluginTest(WindowsArtifactPreprocessorPluginTestCase):
  """Tests for the Windows code page plugin."""

  def testParseValueData(self):
    """Tests the _ParseValueData function."""
    test_file_path = self._GetTestFilePath(['SYSTEM'])
    self._SkipIfPathNotExists(test_file_path)

    storage_writer = self._CreateTestStorageWriter()

    plugin = windows.WindowsCodePagePlugin()
    test_mediator = self._RunPreprocessorPluginOnWindowsRegistryValueSystem(
        storage_writer, plugin)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'preprocessing_warning')
    self.assertEqual(number_of_warnings, 0)

    self.assertEqual(test_mediator.code_page, 'cp1252')


class WindowsEventLogPublishersPluginTest(
    WindowsArtifactPreprocessorPluginTestCase):
  """Tests for the Windows Event Log publishers plugin."""

  def testParseValueData(self):
    """Tests the _ParseValueData function."""
    test_file_path = self._GetTestFilePath(['SOFTWARE'])
    self._SkipIfPathNotExists(test_file_path)

    storage_writer = self._CreateTestStorageWriter()

    plugin = windows.WindowsEventLogPublishersPlugin()
    self._RunPreprocessorPluginOnWindowsRegistryValueSoftware(
        storage_writer, plugin)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'preprocessing_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_artifacts = storage_writer.GetNumberOfAttributeContainers(
        'windows_eventlog_provider')
    self.assertEqual(number_of_artifacts, 438)


class WindowsEventLogSourcesPluginTest(
    WindowsArtifactPreprocessorPluginTestCase):
  """Tests for the Windows Event Log sources plugin."""

  def testParseValueData(self):
    """Tests the _ParseValueData function."""
    test_file_path = self._GetTestFilePath(['SYSTEM'])
    self._SkipIfPathNotExists(test_file_path)

    storage_writer = self._CreateTestStorageWriter()

    plugin = windows.WindowsEventLogSourcesPlugin()
    self._RunPreprocessorPluginOnWindowsRegistryValueSystem(
        storage_writer, plugin)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'preprocessing_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_artifacts = storage_writer.GetNumberOfAttributeContainers(
        'windows_eventlog_provider')
    self.assertEqual(number_of_artifacts, 374)


class WindowsHostnamePluginTest(WindowsArtifactPreprocessorPluginTestCase):
  """Tests for the Windows hostname plugin."""

  # pylint: disable=protected-access

  def testParseValueData(self):
    """Tests the _ParseValueData function."""
    test_file_path = self._GetTestFilePath(['SYSTEM'])
    self._SkipIfPathNotExists(test_file_path)

    storage_writer = self._CreateTestStorageWriter()

    plugin = windows.WindowsHostnamePlugin()
    test_mediator = self._RunPreprocessorPluginOnWindowsRegistryValueSystem(
        storage_writer, plugin)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'preprocessing_warning')
    self.assertEqual(number_of_warnings, 0)

    self.assertEqual(test_mediator.hostname.name, 'WKS-WIN732BITA')

    value_data = ['MyHost', '']
    plugin._ParseValueData(test_mediator, value_data)


class WindowsLanguagePlugin(WindowsArtifactPreprocessorPluginTestCase):
  """Tests for the Windows language plugin."""

  def testParseValueData(self):
    """Tests the _ParseValueData function."""
    test_file_path = self._GetTestFilePath(['SYSTEM'])
    self._SkipIfPathNotExists(test_file_path)

    storage_writer = self._CreateTestStorageWriter()

    plugin = windows.WindowsLanguagePlugin()
    test_mediator = self._RunPreprocessorPluginOnWindowsRegistryValueSystem(
        storage_writer, plugin)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'preprocessing_warning')
    self.assertEqual(number_of_warnings, 0)

    self.assertEqual(test_mediator.language, 'en-US')


class WindowsMountedDevicesPluginTest(
    WindowsArtifactPreprocessorPluginTestCase):
  """Tests for the Windows mounted devices plugin."""

  # pylint: disable=protected-access

  def testParseValueData(self):
    """Tests the _ParseValueData function."""
    test_file_path = self._GetTestFilePath(['SYSTEM'])
    self._SkipIfPathNotExists(test_file_path)

    storage_writer = self._CreateTestStorageWriter()

    plugin = windows.WindowsMountedDevicesPlugin()
    self._RunPreprocessorPluginOnWindowsRegistryValueSystem(
        storage_writer, plugin)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'preprocessing_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_windows_mounted_devices = (
        storage_writer._attribute_containers_counter['windows_mounted_device'])
    self.assertEqual(number_of_windows_mounted_devices, 11)

    attribute_containers = list(storage_writer.GetAttributeContainers(
        'windows_mounted_device'))
    self.assertEqual(len(attribute_containers), 11)

    mounted_device_artifact = attribute_containers[0]
    self.assertEqual(mounted_device_artifact.identifier, '\\DosDevices\\C:')
    self.assertEqual(mounted_device_artifact.disk_identity, 0x5cbea03e)
    self.assertEqual(mounted_device_artifact.partition_offset, 1048576)


class WindowsProgramDataEnvironmentVariablePluginTest(
    WindowsArtifactPreprocessorPluginTestCase):
  """Tests for the %ProgramData% environment variable plugin."""

  def testParseValueData(self):
    """Tests the _ParseValueData function."""
    test_file_path = self._GetTestFilePath(['SOFTWARE'])
    self._SkipIfPathNotExists(test_file_path)

    storage_writer = self._CreateTestStorageWriter()

    plugin = windows.WindowsProgramDataEnvironmentVariablePlugin()
    self._RunPreprocessorPluginOnWindowsRegistryValueSoftware(
        storage_writer, plugin)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'preprocessing_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_artifacts = storage_writer.GetNumberOfAttributeContainers(
        'environment_variable')
    self.assertEqual(number_of_artifacts, 1)

    environment_variable = storage_writer.GetAttributeContainerByIndex(
        'environment_variable', 0)
    self.assertIsNotNone(environment_variable)
    self.assertEqual(environment_variable.name, 'programdata')
    self.assertEqual(environment_variable.value, '%SystemDrive%\\ProgramData')


class WindowsProgramDataKnowledgeBasePluginTest(
    test_lib.ArtifactPreprocessorPluginTestCase):
  """Tests for the programdata knowledge base value plugin."""

  # pylint: disable=protected-access

  def testCollect(self):
    """Tests the Collect function."""
    plugin = windows.WindowsProgramDataKnowledgeBasePlugin()

    storage_writer = self._CreateTestStorageWriter()
    test_mediator = mediator.PreprocessMediator(storage_writer)

    plugin.Collect(test_mediator)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'preprocessing_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_artifacts = storage_writer.GetNumberOfAttributeContainers(
        'environment_variable')
    self.assertEqual(number_of_artifacts, 0)

  def testCollectWithAllUsersProfile(self):
    """Tests the Collect function with the %AllUsersProfile% variable."""
    plugin = windows.WindowsProgramDataKnowledgeBasePlugin()

    storage_writer = self._CreateTestStorageWriter()
    test_mediator = mediator.PreprocessMediator(storage_writer)

    environment_variable = artifacts.EnvironmentVariableArtifact(
        case_sensitive=False, name='allusersprofile',
        value='C:\\Documents and Settings\\All Users')

    test_mediator._environment_variables['ALLUSERSPROFILE'] = (
        environment_variable)

    plugin.Collect(test_mediator)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'preprocessing_warning')
    self.assertEqual(number_of_warnings, 0)

    # The %ProgramData% environment variable is derived from
    # the %AllUsersProfile% environment variable.
    number_of_artifacts = storage_writer.GetNumberOfAttributeContainers(
        'environment_variable')
    self.assertEqual(number_of_artifacts, 1)

    environment_variable = storage_writer.GetAttributeContainerByIndex(
        'environment_variable', 0)
    self.assertIsNotNone(environment_variable)
    self.assertEqual(environment_variable.name, 'programdata')
    self.assertEqual(
        environment_variable.value, 'C:\\Documents and Settings\\All Users')

  def testCollectWithProgramData(self):
    """Tests the Collect function with the %ProgramData% variable."""
    plugin = windows.WindowsProgramDataKnowledgeBasePlugin()

    storage_writer = self._CreateTestStorageWriter()
    test_mediator = mediator.PreprocessMediator(storage_writer)

    environment_variable = artifacts.EnvironmentVariableArtifact(
        case_sensitive=False, name='programdata',
        value='%SystemDrive%\\ProgramData')

    test_mediator._environment_variables['PROGRAMDATA'] = environment_variable

    plugin.Collect(test_mediator)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'preprocessing_warning')
    self.assertEqual(number_of_warnings, 0)

    # The %ProgramData% environment variable is already set in
    # the knowledge base and should not be created.
    number_of_artifacts = storage_writer.GetNumberOfAttributeContainers(
        'environment_variable')
    self.assertEqual(number_of_artifacts, 0)


class WindowsProgramFilesEnvironmentVariablePluginTest(
    WindowsArtifactPreprocessorPluginTestCase):
  """Tests for the %ProgramFiles% environment variable plugin."""

  def testParseValueData(self):
    """Tests the _ParseValueData function."""
    test_file_path = self._GetTestFilePath(['SOFTWARE'])
    self._SkipIfPathNotExists(test_file_path)

    storage_writer = self._CreateTestStorageWriter()

    plugin = windows.WindowsProgramFilesEnvironmentVariablePlugin()
    self._RunPreprocessorPluginOnWindowsRegistryValueSoftware(
        storage_writer, plugin)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'preprocessing_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_artifacts = storage_writer.GetNumberOfAttributeContainers(
        'environment_variable')
    self.assertEqual(number_of_artifacts, 1)

    environment_variable = storage_writer.GetAttributeContainerByIndex(
        'environment_variable', 0)
    self.assertIsNotNone(environment_variable)
    self.assertEqual(environment_variable.name, 'programfiles')
    self.assertEqual(environment_variable.value, 'C:\\Program Files')


class WindowsProgramFilesX86EnvironmentVariablePluginTest(
    WindowsArtifactPreprocessorPluginTestCase):
  """Tests for the %ProgramFilesX86% environment variable plugin."""

  def testParseValueData(self):
    """Tests the _ParseValueData function."""
    test_file_path = self._GetTestFilePath(['SOFTWARE'])
    self._SkipIfPathNotExists(test_file_path)

    storage_writer = self._CreateTestStorageWriter()

    plugin = windows.WindowsProgramFilesX86EnvironmentVariablePlugin()
    self._RunPreprocessorPluginOnWindowsRegistryValueSoftware(
        storage_writer, plugin)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'preprocessing_warning')
    self.assertEqual(number_of_warnings, 0)

    # The test SOFTWARE Registry file does not contain a value for
    # the Program Files X86 path.
    number_of_artifacts = storage_writer.GetNumberOfAttributeContainers(
        'environment_variable')
    self.assertEqual(number_of_artifacts, 0)


class WindowsSystemRootEnvironmentVariablePluginTest(
    test_lib.ArtifactPreprocessorPluginTestCase):
  """Tests for the %SystemRoot% environment variable plugin."""

  _FILE_DATA = b'regf'

  def testParsePathSpecification(self):
    """Tests the _ParsePathSpecification function."""
    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    file_system_builder.AddFile(
        '/Windows/System32/config/SYSTEM', self._FILE_DATA)

    mount_point = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_FAKE, location='/')

    storage_writer = self._CreateTestStorageWriter()
    plugin = windows.WindowsSystemRootEnvironmentVariablePlugin()
    self._RunPreprocessorPluginOnFileSystem(
        file_system_builder.file_system, mount_point, storage_writer, plugin)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'preprocessing_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_artifacts = storage_writer.GetNumberOfAttributeContainers(
        'environment_variable')
    self.assertEqual(number_of_artifacts, 1)

    environment_variable = storage_writer.GetAttributeContainerByIndex(
        'environment_variable', 0)
    self.assertIsNotNone(environment_variable)
    self.assertEqual(environment_variable.name, 'systemroot')
    self.assertEqual(environment_variable.value, '\\Windows')


class WindowsServicesAndDriversPluginTest(
    WindowsArtifactPreprocessorPluginTestCase):
  """Tests for the Windows service (and driver) configurations plugin."""

  def testParseValueData(self):
    """Tests the _ParseValueData function."""
    test_file_path = self._GetTestFilePath(['SYSTEM'])
    self._SkipIfPathNotExists(test_file_path)

    storage_writer = self._CreateTestStorageWriter()

    plugin = windows.WindowsServicesAndDriversPlugin()
    self._RunPreprocessorPluginOnWindowsRegistryValueSystem(
        storage_writer, plugin)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'preprocessing_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_artifacts = storage_writer.GetNumberOfAttributeContainers(
        'windows_service_configuration')
    self.assertEqual(number_of_artifacts, 416)


class WindowsSystemProductPluginTest(
    WindowsArtifactPreprocessorPluginTestCase):
  """Tests for the system product information plugin."""

  def testParseValueData(self):
    """Tests the _ParseValueData function."""
    test_file_path = self._GetTestFilePath(['SOFTWARE'])
    self._SkipIfPathNotExists(test_file_path)

    storage_writer = self._CreateTestStorageWriter()

    plugin = windows.WindowsSystemProductPlugin()
    test_mediator = self._RunPreprocessorPluginOnWindowsRegistryValueSoftware(
        storage_writer, plugin)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'preprocessing_warning')
    self.assertEqual(number_of_warnings, 0)

    system_product = test_mediator.GetValue('operating_system_product')
    self.assertEqual(system_product, 'Windows 7 Ultimate')


class WindowsSystemVersionPluginTest(
    WindowsArtifactPreprocessorPluginTestCase):
  """Tests for the system version information plugin."""

  def testParseValueData(self):
    """Tests the _ParseValueData function."""
    test_file_path = self._GetTestFilePath(['SOFTWARE'])
    self._SkipIfPathNotExists(test_file_path)

    storage_writer = self._CreateTestStorageWriter()

    plugin = windows.WindowsSystemVersionPlugin()
    test_mediator = self._RunPreprocessorPluginOnWindowsRegistryValueSoftware(
        storage_writer, plugin)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'preprocessing_warning')
    self.assertEqual(number_of_warnings, 0)

    system_version = test_mediator.GetValue('operating_system_version')
    self.assertEqual(system_version, '6.1')


class WindowsTimeZonePluginTest(WindowsArtifactPreprocessorPluginTestCase):
  """Tests for the time zone plugin."""

  def testParseValueData(self):
    """Tests the _ParseValueData function."""
    test_file_path = self._GetTestFilePath(['SYSTEM'])
    self._SkipIfPathNotExists(test_file_path)

    storage_writer = self._CreateTestStorageWriter()

    plugin = windows.WindowsTimeZonePlugin()
    test_mediator = self._RunPreprocessorPluginOnWindowsRegistryValueSystem(
        storage_writer, plugin)

    # Unable to map: "@tzres.dll,-112" to time zone with error: Unsupported
    # time zone: @tzres.dll,-112
    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'preprocessing_warning')
    self.assertEqual(number_of_warnings, 1)

    self.assertEqual(test_mediator.time_zone.zone, 'America/New_York')


class WindowsUserAccountsPluginTest(WindowsArtifactPreprocessorPluginTestCase):
  """Tests for the Windows user accounts artifacts mapping."""

  # pylint: disable=protected-access

  def _CreateTestKey(self):
    """Creates Registry keys and values for testing.

    Returns:
      dfwinreg.WinRegistryKey: a Windows Registry key.
    """
    key_path_prefix = (
        'HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows NT\\'
        'CurrentVersion')
    relative_key_path = 'Microsoft\\Windows NT\\CurrentVersion'
    registry_key = dfwinreg_fake.FakeWinRegistryKey(
        'ProfileList', key_path_prefix=key_path_prefix,
        relative_key_path=relative_key_path)

    # Setup user profile sub key.

    profile_key_name = 'S-1-5-21-2036804247-3058324640-2116585241-1114'
    profile_key = dfwinreg_fake.FakeWinRegistryKey(profile_key_name)
    registry_key.AddSubkey(profile_key_name, profile_key)

    return profile_key

  def testParseKey(self):
    """Tests the _ParseKey function."""
    test_file_entry = self._GetTestFileEntry(['SOFTWARE'])
    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)

    registry_key = win_registry.GetKeyByPath((
        'HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows NT\\CurrentVersion\\'
        'ProfileList\\S-1-5-21-2036804247-3058324640-2116585241-1114'))

    plugin = windows.WindowsUserAccountsPlugin()

    storage_writer = self._CreateTestStorageWriter()
    test_mediator = mediator.PreprocessMediator(storage_writer)
    plugin._ParseKey(test_mediator, registry_key, 'ProfileImagePath')

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'preprocessing_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_artifacts = storage_writer.GetNumberOfAttributeContainers(
        'user_account')
    self.assertEqual(number_of_artifacts, 1)

    user_account = storage_writer.GetAttributeContainerByIndex(
       'user_account', 0)

    self.assertEqual(
        user_account.identifier,
        'S-1-5-21-2036804247-3058324640-2116585241-1114')
    self.assertEqual(user_account.username, 'rsydow')
    self.assertEqual(user_account.user_directory, 'C:\\Users\\rsydow')

    # Test key with empty ProfileImagePath value.
    registry_key = self._CreateTestKey()

    storage_writer = self._CreateTestStorageWriter()
    test_mediator = mediator.PreprocessMediator(storage_writer)
    plugin._ParseKey(test_mediator, registry_key, 'ProfileImagePath')

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'preprocessing_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_artifacts = storage_writer.GetNumberOfAttributeContainers(
        'user_account')
    self.assertEqual(number_of_artifacts, 1)

    user_account = storage_writer.GetAttributeContainerByIndex(
       'user_account', 0)

    self.assertEqual(
        user_account.identifier,
        'S-1-5-21-2036804247-3058324640-2116585241-1114')
    self.assertIsNone(user_account.username)
    self.assertIsNone(user_account.user_directory)


class WindowsWinDirEnvironmentVariablePluginTest(
    test_lib.ArtifactPreprocessorPluginTestCase):
  """Tests for the %WinDir% environment variable plugin."""

  _FILE_DATA = b'regf'

  def testParsePathSpecification(self):
    """Tests the _ParsePathSpecification function."""
    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    file_system_builder.AddFile(
        '/Windows/System32/config/SYSTEM', self._FILE_DATA)

    mount_point = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_FAKE, location='/')

    storage_writer = self._CreateTestStorageWriter()
    plugin = windows.WindowsWinDirEnvironmentVariablePlugin()
    self._RunPreprocessorPluginOnFileSystem(
        file_system_builder.file_system, mount_point, storage_writer, plugin)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'preprocessing_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_artifacts = storage_writer.GetNumberOfAttributeContainers(
        'environment_variable')
    self.assertEqual(number_of_artifacts, 1)

    environment_variable = storage_writer.GetAttributeContainerByIndex(
        'environment_variable', 0)
    self.assertIsNotNone(environment_variable)
    self.assertEqual(environment_variable.name, 'windir')
    self.assertEqual(environment_variable.value, '\\Windows')


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Parser for fish history files."""

import os
import re

import yaml

from dfdatetime import posix_time as dfdatetime_posix_time

from dfvfs.helpers import text_file

from plaso.containers import events
from plaso.lib import errors
from plaso.parsers import manager
from plaso.parsers import interface


class FishHistoryEventData(events.EventData):
  """Fish history log event data.

  Attributes:
    command (str): command that was executed.
    written_time (dfdatetime.DateTimeValues): date and time the entry was
        written.
  """

  DATA_TYPE = 'fish:history:entry'

  def __init__(self):
    """Initializes event data."""
    super(FishHistoryEventData, self).__init__(data_type=self.DATA_TYPE)
    self.command = None
    self.written_time = None


class FishHistoryParser(interface.FileObjectParser):
  """Parses events from Fish history files."""

  NAME = 'fish_history'
  DATA_FORMAT = 'Fish history file'

  _ENCODING = 'utf-8'

  # 50 MiB is the maximum supported fish history file size.
  _MAXIMUM_FISH_HISTORY_FILE_SIZE = 50 * 1024 * 1024

  _FILENAME = 'fish_history'

  _YAML_FORMAT_RE_1 = re.compile(r'^- cmd: \S+')
  _YAML_FORMAT_RE_2 = re.compile(r'  when: [0-9]{9}')

  def ParseFileObject(self, parser_mediator, file_object):
    """Parses a fish history file from a file-like object.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      file_object (dfvfs.FileIO): a file-like object.

    Raises:
      WrongParser: when the file cannot be parsed.
    """
    filename = parser_mediator.GetFilename()

    if filename != self._FILENAME:
      raise errors.WrongParser('Not a Fish history file.')

    text_file_object = text_file.TextFile(file_object, encoding='utf-8')

    header_line_1 = text_file_object.readline()
    header_line_2 = text_file_object.readline()
    if (not self._YAML_FORMAT_RE_1.match(header_line_1) or
        not self._YAML_FORMAT_RE_2.match(header_line_2)):
      raise errors.WrongParser('Not a valid fish history file.')

    file_size = file_object.get_size()
    if file_size > self._MAXIMUM_FISH_HISTORY_FILE_SIZE:
      parser_mediator.ProduceExtractionWarning(
          'Fish history file size: {0:d} exceeds maxmimum'.format(file_size))
      return

    file_object.seek(0, os.SEEK_SET)

    try:
      fish_history = yaml.safe_load(file_object)
    except yaml.YAMLError as exception:
      parser_mediator.ProduceExtractionWarning(
          'Error reading YAML with error: {0:s}'.format(exception))
      return

    for entry_index, history_entry in enumerate(fish_history):
      command = history_entry.get('cmd', None)
      timestamp = history_entry.get('when', None)

      if None in (command, timestamp):
        parser_mediator.ProduceExtractionWarning(
            'Unsupported history entry: {0:d}'.format(entry_index))
        continue

      if not isinstance(timestamp, int):
        try:
          timestamp = int(timestamp, 10)
        except (TypeError, ValueError):
          parser_mediator.ProduceExtractionWarning(
              'Unsupported timestamp: {0!s} in history entry: {1:s}'.format(
                  timestamp, entry_index))
          continue

      event_data = FishHistoryEventData()
      event_data.command = command
      event_data.written_time = dfdatetime_posix_time.PosixTime(
          timestamp=timestamp)

      parser_mediator.ProduceEventData(event_data)


manager.ParsersManager.RegisterParser(FishHistoryParser)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for Basic Security Module (BSM) file parser."""

import unittest

from plaso.parsers import bsm

from tests.parsers import test_lib


class MacOSBSMParserTest(test_lib.ParserTestCase):
  """Tests for Basic Security Module (BSM) file parser."""

  def testParse(self):
    """Tests the Parse function on a MacOS BSM file."""
    parser = bsm.BSMParser()
    storage_writer = self._ParseFile(['apple.bsm'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 54)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_extra_tokens = [
        {'AUT_TEXT': {
            'text': 'launchctl::Audit recovery'}},
        {'AUT_PATH': {
            'path': '/var/audit/20131104171720.crash_recovery'}},
        {'AUT_RETURN32': {
            'call_status': 0,
            'error': 'Success',
            'token_status': 0}}]

    expected_event_values = {
        'data_type': 'bsm:entry',
        'event_type': 45029,
        'extra_tokens': expected_extra_tokens,
        'return_value': (
            '{\'error\': \'Success\', \'token_status\': 0, '
            '\'call_status\': 0}'),
        'written_time': '2013-11-04T18:36:20.000381+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


class OpenBSMParserTest(test_lib.ParserTestCase):
  """Tests for Basic Security Module (BSM) file parser."""

  def testParse(self):
    """Tests the Parse function on a "generic" BSM file."""
    parser = bsm.BSMParser()
    storage_writer = self._ParseFile(['openbsm.bsm'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 50)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_extra_tokens = [{
        'AUT_ARG32': {
            'is': 2882400000,
            'num_arg': 3,
            'string': 'test_arg32_token'}}]

    expected_event_values = {
        'data_type': 'bsm:entry',
        'event_type': 0,
        'extra_tokens': expected_extra_tokens,
        'return_value': None,
        'written_time': '2008-12-28T15:12:18.000131+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the LastShutdown value plugin."""

import unittest

from plaso.parsers.winreg_plugins import shutdown

from tests.parsers.winreg_plugins import test_lib


class ShutdownWindowsRegistryPluginTest(test_lib.RegistryPluginTestCase):
  """Tests for the LastShutdown value plugin."""

  def testFilters(self):
    """Tests the FILTERS class attribute."""
    plugin = shutdown.ShutdownWindowsRegistryPlugin()

    self._AssertFiltersOnKeyPath(
        plugin, 'HKEY_LOCAL_MACHINE\\System', 'ControlSet001\\Control\\Windows')

    self._AssertNotFiltersOnKeyPath(
        plugin, 'HKEY_LOCAL_MACHINE\\System', 'Bogus')

  def testProcess(self):
    """Tests the Process function."""
    test_file_entry = self._GetTestFileEntry(['SYSTEM'])
    key_path = 'HKEY_LOCAL_MACHINE\\System\\ControlSet001\\Control\\Windows'

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)

    plugin = shutdown.ShutdownWindowsRegistryPlugin()
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 2)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'windows:registry:shutdown',
        'key_path': key_path,
        'last_shutdown_time': '2012-04-04T01:58:40.8392499+00:00',
        'value_name': 'ShutdownTime'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the MSIE typed URLs Windows Registry plugin."""

import unittest

from plaso.parsers.winreg_plugins import typedurls

from tests.parsers.winreg_plugins import test_lib


class MsieTypedURLsPluginTest(test_lib.RegistryPluginTestCase):
  """Tests for the MSIE typed URLs Windows Registry plugin."""

  def testFilters(self):
    """Tests the FILTERS class attribute."""
    plugin = typedurls.TypedURLsPlugin()

    self._AssertFiltersOnKeyPath(plugin, 'HKEY_CURRENT_USER', (
        'Software\\Microsoft\\Internet Explorer\\TypedURLs'))

    self._AssertFiltersOnKeyPath(plugin, 'HKEY_CURRENT_USER', (
        'Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\TypedPaths'))

    self._AssertNotFiltersOnKeyPath(plugin, 'HKEY_CURRENT_USER', 'Bogus')

  def testProcess(self):
    """Tests the Process function."""
    test_file_entry = self._GetTestFileEntry(['NTUSER-WIN7.DAT'])
    key_path = (
        'HKEY_CURRENT_USER\\Software\\Microsoft\\Internet Explorer\\'
        'TypedURLs')

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)

    plugin = typedurls.TypedURLsPlugin()
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'windows:registry:typedurls',
        'entries': (
            'url1: http://cnn.com/ '
            'url2: http://twitter.com/ '
            'url3: http://linkedin.com/ '
            'url4: http://tweetdeck.com/ '
            'url5: mozilla '
            'url6: http://google.com/ '
            'url7: http://controller.shieldbase.local/certsrv/ '
            'url8: http://controller.shieldbase.local/ '
            'url9: http://www.stark-research-labs.com/ '
            'url10: http://www.adobe.com/ '
            'url11: http://www.google.com/ '
            'url12: http://www.firefox.com/ '
            'url13: http://go.microsoft.com/fwlink/?LinkId=69157'),
        'key_path': key_path,
        'last_written_time': '2012-03-12T21:23:53.3077499+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


class TypedPathsPluginTest(test_lib.RegistryPluginTestCase):
  """Tests for the typed paths Windows Registry plugin."""

  def testProcess(self):
    """Tests the Process function."""
    test_file_entry = self._GetTestFileEntry(['NTUSER-WIN7.DAT'])
    key_path = (
        'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
        'Explorer\\TypedPaths')

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)

    plugin = typedurls.TypedURLsPlugin()
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'windows:registry:typedurls',
        'entries': 'url1: \\\\controller',
        'key_path': key_path,
        'last_written_time': '2010-11-10T07:58:15.8116250+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Google Chrome Preferences file event formatter."""

import unittest

from plaso.formatters import chrome_preferences

from tests.formatters import test_lib


class ChromePreferencesPrimaryURLFormatterHelperTest(
    test_lib.EventFormatterTestCase):
  """Tests for the Google Chrome preferences primary URL formatter helper."""

  def testFormatEventValues(self):
    """Tests the FormatEventValues function."""
    formatter_helper = (
        chrome_preferences.ChromePreferencesPrimaryURLFormatterHelper())

    output_mediator = self._CreateOutputMediator()

    event_values = {'primary_url': 'https://example.com'}
    formatter_helper.FormatEventValues(output_mediator, event_values)
    self.assertEqual(event_values['primary_url'], 'https://example.com')

    event_values = {'primary_url': ''}
    formatter_helper.FormatEventValues(output_mediator, event_values)
    self.assertEqual(event_values['primary_url'], 'local file')

    event_values = {'primary_url': None}
    formatter_helper.FormatEventValues(output_mediator, event_values)
    self.assertIsNone(event_values['primary_url'])


class ChromePreferencesSecondaryURLFormatterHelperTest(
    test_lib.EventFormatterTestCase):
  """Tests for the Google Chrome preferences secondary URL formatter helper."""

  def testFormatEventValues(self):
    """Tests the FormatEventValues function."""
    formatter_helper = (
        chrome_preferences.ChromePreferencesSecondaryURLFormatterHelper())

    output_mediator = self._CreateOutputMediator()

    event_values = {
        'primary_url': 'https://example.com',
        'secondary_url': 'https://anotherexample.com'}
    formatter_helper.FormatEventValues(output_mediator, event_values)
    self.assertEqual(
        event_values['secondary_url'], 'https://anotherexample.com')

    event_values = {
        'primary_url': 'https://example.com',
        'secondary_url': 'https://example.com'}
    formatter_helper.FormatEventValues(output_mediator, event_values)
    self.assertIsNone(event_values['secondary_url'])

    event_values = {
        'primary_url': 'https://example.com',
        'secondary_url': ''}
    formatter_helper.FormatEventValues(output_mediator, event_values)
    self.assertEqual(event_values['secondary_url'], 'local file')

    event_values = {
        'primary_url': 'https://example.com',
        'secondary_url': None}
    formatter_helper.FormatEventValues(output_mediator, event_values)
    self.assertIsNone(event_values['secondary_url'])


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The bloom database analysis plugin CLI arguments helper."""

from plaso.analysis import bloom
from plaso.cli.helpers import interface
from plaso.cli.helpers import manager
from plaso.lib import errors


class BloomAnalysisArgumentsHelper(interface.ArgumentsHelper):
  """Bloom database analysis plugin CLI arguments helper."""

  NAME = 'bloom'
  CATEGORY = 'analysis'
  DESCRIPTION = 'Argument helper for the bloom database analysis plugin.'

  _DEFAULT_BLOOM_DATABASE_PATH = "hashlookup-full.bloom"
  _DEFAULT_HASH = 'sha1'
  _DEFAULT_LABEL = bloom.BloomAnalysisPlugin.DEFAULT_LABEL
  _SUPPORTED_HASHES = sorted(
      bloom.BloomAnalysisPlugin.SUPPORTED_HASHES)

  @classmethod
  def AddArguments(cls, argument_group):
    """Adds command line arguments the helper supports to an argument group.

    This function takes an argument parser or an argument group object and adds
    to it all the command line arguments this helper supports.

    Args:
      argument_group (argparse._ArgumentGroup|argparse.ArgumentParser): group
          to append arguments to.
    """
    argument_group.add_argument(
        '--bloom-file', '--bloom_file', dest='bloom_file', type=str,
        action='store', default=cls._DEFAULT_BLOOM_DATABASE_PATH,
        metavar='PATH', help=(
            f'Path to the bloom database file, the default is: '
            f'{cls._DEFAULT_BLOOM_DATABASE_PATH:s}'))

    supported_hashes = ', '.join(cls._SUPPORTED_HASHES)
    argument_group.add_argument(
        '--bloom-hash', '--bloom_hash', dest='bloom_hash', type=str,
        action='store', choices=cls._SUPPORTED_HASHES,
        default=cls._DEFAULT_HASH, metavar='HASH', help=(
            f'Type of hash to use to query the bloom database file (note that '
            f'hash values must be stored in upper case), the default is: '
            f'{cls._DEFAULT_HASH:s}. Supported options: {supported_hashes:s}.'))

    argument_group.add_argument(
        '--bloom-label', '--bloom_label', dest='bloom_label', type=str,
        action='store', default=cls._DEFAULT_LABEL, metavar='LABEL', help=(
            f'Label to apply to events, the default is: '
            f'{cls._DEFAULT_LABEL:s}.'))

  @classmethod
  def ParseOptions(cls, options, analysis_plugin):  # pylint: disable=arguments-renamed
    """Parses and validates options.

    Args:
      options (argparse.Namespace): parser options object.
      analysis_plugin (BloomAnalysisPlugin): analysis plugin
        to configure.

    Raises:
      BadConfigObject: when the analysis plugin is the wrong type.
      BadConfigOption: when unable to load the bloom database file.
    """
    if not isinstance(analysis_plugin, bloom.BloomAnalysisPlugin):
      raise errors.BadConfigObject(
          'Analysis plugin is not an instance of BloomAnalysisPlugin')

    label = cls._ParseStringOption(
        options, 'bloom_label', default_value=cls._DEFAULT_LABEL)
    analysis_plugin.SetLabel(label)

    lookup_hash = cls._ParseStringOption(
        options, 'bloom_hash', default_value=cls._DEFAULT_HASH)
    analysis_plugin.SetLookupHash(lookup_hash)

    bloom_file = cls._ParseStringOption(
        options, 'bloom_file',
        default_value=cls._DEFAULT_BLOOM_DATABASE_PATH)
    analysis_plugin.SetBloomDatabasePath(bloom_file)
    if not analysis_plugin.TestLoading():
      raise errors.BadConfigOption(
          f'Unable to load bloom database: {bloom_file:s}')


manager.ArgumentHelperManager.RegisterHelper(BloomAnalysisArgumentsHelper)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the MacOS Duet/KnowledgeC database."""

import unittest

from plaso.parsers.sqlite_plugins import macos_knowledgec

from tests.parsers.sqlite_plugins import test_lib


class MacOSKnowledgecTest(test_lib.SQLitePluginTestCase):
  """Tests for the MacOS Duet/KnowledgeC database."""

  def testProcessHighSierra(self):
    """Tests the Process function on a MacOS 10.13 database."""
    plugin = macos_knowledgec.MacOSKnowledgeCPlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['mac_knowledgec-10.13.db'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 17)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'bundle_identifier': 'com.apple.Installer-Progress',
        'creation_time': '2019-02-10T16:59:58.860664+00:00',
        'data_type': 'macos:knowledgec:application',
        'duration': 1,
        'end_time': '2019-02-10T16:59:58.000000+00:00',
        'start_time': '2019-02-10T16:59:57.000000+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessMojave(self):
    """Tests the Process function on a MacOS 10.14 database."""
    plugin = macos_knowledgec.MacOSKnowledgeCPlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['mac_knowledgec-10.14.db'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 77)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'bundle_identifier': 'com.apple.Terminal',
        'creation_time': '2019-05-08T13:57:30.668998+00:00',
        'data_type': 'macos:knowledgec:application',
        'duration': 1041,
        'end_time': '2019-05-08T13:57:30.000000+00:00',
        'start_time': '2019-05-08T13:40:09.000000+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 75)
    self.CheckEventData(event_data, expected_event_values)

    expected_event_values = {
        'creation_time': '2019-05-08T13:57:20.626870+00:00',
        'data_type': 'macos:knowledgec:safari',
        'duration': 0,
        'end_time': '2019-05-08T13:57:20.000000+00:00',
        'start_time': '2019-05-08T13:57:20.000000+00:00',
        'title': 'Instagram',
        'url': 'https://www.instagram.com/'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 70)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the bloom analysis plugin."""

import collections
import unittest

from dfvfs.path import fake_path_spec

try:
  import flor
  from plaso.analysis import bloom
except ModuleNotFoundError:
  flor = None

from plaso.containers import events
from plaso.containers import reports
from plaso.lib import definitions

from tests.analysis import test_lib


@unittest.skipIf(flor is None, 'missing flor support')
class BloomTest(test_lib.AnalysisPluginTestCase):
  """Tests for the bloom database analysis plugin."""

  _EVENT_1_HASH = '1e5f96de17b84a94f69e52a24b2942c4eca11bc9'
  _EVENT_2_HASH = 'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'

  _TEST_EVENTS = [
      {'_parser_chain': 'filestat',
       'data_type': 'fs:stat',
       'path_spec': fake_path_spec.FakePathSpec(
           location='C:\\WINDOWS\\system32\\know.exe'),
       'sha256_hash': _EVENT_1_HASH,
       'timestamp': '2015-01-01 17:00:00',
       'timestamp_desc': definitions.TIME_DESCRIPTION_CREATION},
      {'_parser_chain': 'filestat',
       'data_type': 'fs:stat:ntfs',
       'path_spec': fake_path_spec.FakePathSpec(
           location='C:\\WINDOWS\\system32\\unknown.exe'),
       'sha256_hash': _EVENT_2_HASH,
       'timestamp': '2016-01-01 17:00:00',
       'timestamp_desc': definitions.TIME_DESCRIPTION_CREATION}]

  def testExamineEventAndCompileReport(self):
    """Tests the ExamineEvent and CompileReport functions."""
    plugin = bloom.BloomAnalysisPlugin()
    # this file contains only one hash : _EVENT_1_HASH
    plugin.SetBloomDatabasePath('test_data/plaso.bloom')
    plugin.SetLabel('bloom_present')

    storage_writer = self._AnalyzeEvents(self._TEST_EVENTS, plugin)

    number_of_reports = storage_writer.GetNumberOfAttributeContainers(
        'analysis_report')
    self.assertEqual(number_of_reports, 1)

    analysis_report = storage_writer.GetAttributeContainerByIndex(
        reports.AnalysisReport.CONTAINER_TYPE, 0)
    self.assertIsNotNone(analysis_report)

    self.assertEqual(analysis_report.plugin_name, 'bloom')

    expected_analysis_counter = collections.Counter({
        'bloom_present': 1})
    self.assertEqual(
        analysis_report.analysis_counter, expected_analysis_counter)

    number_of_event_tags = storage_writer.GetNumberOfAttributeContainers(
        'event_tag')
    self.assertEqual(number_of_event_tags, 1)

    labels = []
    for event_tag in storage_writer.GetAttributeContainers(
        events.EventTag.CONTAINER_TYPE):
      labels.extend(event_tag.labels)
    self.assertEqual(len(labels), 1)

    expected_labels = ['bloom_present']
    self.assertEqual(labels, expected_labels)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Session related attribute container definitions."""

import time
import uuid

from acstore.containers import interface
from acstore.containers import manager

import plaso


class Session(interface.AttributeContainer):
  """Session attribute container.

  Attributes:
    aborted (bool): True if the session was aborted.
    artifact_filters (list[str]): Names of artifact definitions that are
        used for filtering file system and Windows Registry key paths.
    command_line_arguments (str): command line arguments.
    completion_time (int): time that the session was completed. Contains the
        number of micro seconds since January 1, 1970, 00:00:00 UTC.
    debug_mode (bool): True if debug mode was enabled.
    enabled_parser_names (list[str]): parser and parser plugin names that
         were enabled.
    filter_file (str): path to a file with find specifications.
    identifier (str): unique identifier of the session.
    parser_filter_expression (str): parser filter expression.
    preferred_codepage (str): preferred codepage.
    preferred_encoding (str): preferred encoding.
    preferred_language (str): preferred language.
    preferred_time_zone (str): preferred time zone.
    preferred_year (int): preferred year.
    product_name (str): name of the product that created the session for
        example "log2timeline".
    product_version (str): version of the product that created the session.
    start_time (int): time that the session was started. Contains the number
        of micro seconds since January 1, 1970, 00:00:00 UTC.
  """

  CONTAINER_TYPE = 'session'

  SCHEMA = {
      'file_entropy': 'str',
      'aborted': 'bool',
      'artifact_filters': 'List[str]',
      'command_line_arguments': 'str',
      'completion_time': 'int',
      'debug_mode': 'bool',
      'enabled_parser_names': 'List[str]',
      'filter_file': 'str',
      'identifier': 'str',
      'parser_filter_expression': 'str',
      'preferred_codepage': 'str',
      'preferred_encoding': 'str',
      'preferred_language': 'str',
      'preferred_time_zone': 'str',
      'preferred_year': 'int',
      'product_name': 'str',
      'product_version': 'str',
      'start_time': 'int'}

  def __init__(self):
    """Initializes a session attribute container."""
    super(Session, self).__init__()
    self.aborted = False
    self.artifact_filters = None
    self.command_line_arguments = None
    self.completion_time = None
    self.debug_mode = False
    self.enabled_parser_names = None
    self.filter_file = None
    self.identifier = '{0:s}'.format(uuid.uuid4().hex)
    self.parser_filter_expression = None
    self.preferred_codepage = None
    self.preferred_encoding = 'utf-8'
    self.preferred_language = None
    self.preferred_time_zone = 'UTC'
    self.preferred_year = None
    self.product_name = 'plaso'
    self.product_version = plaso.__version__
    self.start_time = int(time.time() * 1000000)


manager.AttributeContainersManager.RegisterAttributeContainer(Session)



# -*- coding: utf-8 -*-
"""Parser for Windows NT Registry (REGF) files."""

from dfwinreg import errors as dfwinreg_errors
from dfwinreg import regf as dfwinreg_regf
from dfwinreg import registry as dfwinreg_registry
from dfwinreg import registry_searcher as dfwinreg_registry_searcher

from plaso.engine import artifact_filters
from plaso.lib import specification
from plaso.filters import path_filter
from plaso.parsers import interface
from plaso.parsers import logger
from plaso.parsers import manager


class WinRegistryParser(interface.FileObjectParser):
  """Parses Windows NT Registry (REGF) files."""

  NAME = 'winreg'
  DATA_FORMAT = 'Windows NT Registry (REGF) file'

  _plugin_classes = {}

  _ARTIFACTS_FILTER_HELPER = (
      artifact_filters.ArtifactDefinitionsFiltersHelper)

  _AMCACHE_ROOT_KEY_NAMES = frozenset([
      '{11517b7c-e79d-4e20-961b-75a811715add}',
      '{356c48f6-2fee-e7ef-2a64-39f59ec3be22}'])

  _CONTROL_SET_PREFIX = (
      'HKEY_LOCAL_MACHINE\\System\\ControlSet').lower()

  _NORMALIZED_CONTROL_SET_PREFIX = (
      'HKEY_LOCAL_MACHINE\\System\\CurrentControlSet').lower()

  def __init__(self):
    """Initializes a parser."""
    super(WinRegistryParser, self).__init__()
    self._path_filter = None
    self._plugins_per_key_path = {}
    self._plugins_without_key_paths = []

  def _CanProcessKeyWithPlugin(self, registry_key, plugin):
    """Determines if a plugin can process a Windows Registry key or its values.

    Args:
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.
      plugin (WindowsRegistryPlugin): Windows Registry plugin.

    Returns:
      bool: True if the Registry key can be processed with the plugin.
    """
    for registry_key_filter in plugin.FILTERS:
      # Skip filters that define key paths since they are already
      # checked by the path filter.
      if getattr(registry_key_filter, 'key_paths', []):
        continue

      if registry_key_filter.Match(registry_key):
        return True

    return False

  def EnablePlugins(self, plugin_includes):
    """Enables parser plugins.

    Args:
      plugin_includes (set[str]): names of the plugins to enable, where
          set(['*']) represents all plugins. Note the default plugin, if
          it exists, is always enabled and cannot be disabled.
    """
    self._plugins_per_name = {}
    self._plugins_per_key_path = {}
    self._plugins_without_key_paths = []

    if not self._plugin_classes:
      return

    key_paths = []

    for plugin_name, plugin_class in self._plugin_classes.items():
      if plugin_name == self._default_plugin_name:
        self._default_plugin = plugin_class()
        continue

      if (plugin_includes != self.ALL_PLUGINS and
          plugin_name not in plugin_includes):
        continue

      plugin_object = plugin_class()
      self._plugins_per_name[plugin_name] = plugin_object

      for registry_key_filter in plugin_object.FILTERS:
        plugin_key_paths = getattr(registry_key_filter, 'key_paths', [])
        if (not plugin_key_paths and
            plugin_object not in self._plugins_without_key_paths):
          self._plugins_without_key_paths.append(plugin_object)
          continue

        for plugin_key_path in plugin_key_paths:
          plugin_key_path = plugin_key_path.lower()
          if plugin_key_path in self._plugins_per_key_path:
            logger.warning((
                'Windows Registry key path: {0:s} defined by plugin: {1:s} '
                'already set by plugin: {2:s}').format(
                    plugin_key_path, plugin_object.NAME,
                    self._plugins_per_key_path[plugin_key_path].NAME))
            continue

          self._plugins_per_key_path[plugin_key_path] = plugin_object

          key_paths.append(plugin_key_path)

    self._path_filter = path_filter.PathFilterScanTree(
        key_paths, case_sensitive=False, path_segment_separator='\\')

  @classmethod
  def GetFormatSpecification(cls):
    """Retrieves the format specification."""
    format_specification = specification.FormatSpecification(cls.NAME)
    format_specification.AddNewSignature(b'regf', offset=0)
    return format_specification

  def _ParseKeyWithPlugin(self, parser_mediator, registry_key, plugin):
    """Parses the Registry key with a specific plugin.

    Args:
      parser_mediator (ParserMediator): parser mediator.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.
      plugin (WindowsRegistryPlugin): Windows Registry plugin.
    """
    try:
      plugin.UpdateChainAndProcess(parser_mediator, registry_key)
    except (IOError, dfwinreg_errors.WinRegistryValueError) as exception:
      parser_mediator.ProduceExtractionWarning(
          'in key: {0:s} error: {1!s}'.format(registry_key.path, exception))

  def _NormalizeKeyPath(self, key_path):
    """Normalizes a Windows Registry key path.

    Args:
      key_path (str): Windows Registry key path.

    Returns:
      str: normalized Windows Registry key path.
    """
    normalized_key_path = key_path.lower()
    # The Registry key path should start with:
    # HKEY_LOCAL_MACHINE\System\ControlSet followed by 3 digits
    # which makes 39 characters.
    if (len(normalized_key_path) < 39 or
        not normalized_key_path.startswith(self._CONTROL_SET_PREFIX)):
      return normalized_key_path

    # Key paths that contain ControlSet### must be normalized to
    # CurrentControlSet.
    return ''.join([
        self._NORMALIZED_CONTROL_SET_PREFIX, normalized_key_path[39:]])

  def _ParseKey(self, parser_mediator, registry_key):
    """Parses the Registry key with a specific plugin.

    Args:
      parser_mediator (ParserMediator): parser mediator.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.
    """
    matching_plugin = None

    logger.debug('Parsing Windows Registry key: {0:s}'.format(
        registry_key.path))

    normalized_key_path = self._NormalizeKeyPath(registry_key.path)
    if self._path_filter and self._path_filter.CheckPath(normalized_key_path):
      matching_plugin = self._plugins_per_key_path[normalized_key_path]
    else:
      for plugin in self._plugins_without_key_paths:
        profiling_name = '/'.join([self.NAME, plugin.NAME])

        parser_mediator.SampleFormatCheckStartTiming(profiling_name)

        try:
          if self._CanProcessKeyWithPlugin(registry_key, plugin):
            matching_plugin = plugin
            break

        finally:
          parser_mediator.SampleFormatCheckStopTiming(profiling_name)

    if not matching_plugin:
      matching_plugin = self._default_plugin

    if matching_plugin:
      self._ParseKeyWithPlugin(parser_mediator, registry_key, matching_plugin)

  def _ParseRecurseKeys(self, parser_mediator, registry_key):
    """Parses the Registry keys recursively.

    Args:
      parser_mediator (ParserMediator): parser mediator.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.
    """
    # Note that we do not use dfWinReg generators here to be able to catch
    # exceptions raised by corrupt files.

    self._ParseKey(parser_mediator, registry_key)

    for subkey_index in range(registry_key.number_of_subkeys):
      if parser_mediator.abort:
        break

      try:
        subkey = registry_key.GetSubkeyByIndex(subkey_index)
        self._ParseRecurseKeys(parser_mediator, subkey)

      except IOError as exception:
        parser_mediator.ProduceExtractionWarning(
            'in key: {0:s} error: {1!s}'.format(registry_key.path, exception))

  def _ParseKeysFromFindSpecs(self, parser_mediator, win_registry, find_specs):
    """Parses the Registry keys from FindSpecs.

    Args:
      parser_mediator (ParserMediator): parser mediator.
      win_registry (dfwinreg.WinRegistryKey): root Windows Registry key.
      find_specs (dfwinreg.FindSpecs): Keys to search for.
    """
    searcher = dfwinreg_registry_searcher.WinRegistrySearcher(win_registry)
    for registry_key_path in searcher.Find(find_specs=find_specs):
      if parser_mediator.abort:
        break

      registry_key = searcher.GetKeyByPath(registry_key_path)
      self._ParseKey(parser_mediator, registry_key)

  def ParseFileObject(self, parser_mediator, file_object):
    """Parses a Windows Registry file-like object.

    Args:
      parser_mediator (ParserMediator): parser mediator.
      file_object (dfvfs.FileIO): a file-like object.
    """
    code_page = parser_mediator.GetCodePage()

    registry_file = dfwinreg_regf.REGFWinRegistryFile(ascii_codepage=code_page)

    try:
      registry_file.Open(file_object)
    except IOError as exception:
      parser_mediator.ProduceExtractionWarning(
          'unable to open Windows Registry file with error: {0!s}'.format(
              exception))
      return

    try:
      win_registry = dfwinreg_registry.WinRegistry()

      key_path_prefix = win_registry.GetRegistryFileMapping(registry_file)
      registry_file.SetKeyPathPrefix(key_path_prefix)
      root_key = registry_file.GetRootKey()
      if root_key:
        # For now treat AMCache.hve separately.
        if root_key.name.lower() in self._AMCACHE_ROOT_KEY_NAMES:
          self._ParseRecurseKeys(parser_mediator, root_key)

        elif not parser_mediator.registry_find_specs:
          self._ParseRecurseKeys(parser_mediator, root_key)

        elif not self._ARTIFACTS_FILTER_HELPER.CheckKeyCompatibility(
            key_path_prefix):
          logger.warning((
              'Artifacts filters are not supported for Windows Registry '
              'file with key path prefix: "{0:s}".').format(
                  key_path_prefix))

        else:
          win_registry.MapFile(key_path_prefix, registry_file)
          # Note that win_registry will close the mapped registry_file.
          registry_file = None

          self._ParseKeysFromFindSpecs(
              parser_mediator, win_registry,
              parser_mediator.registry_find_specs)

    except IOError as exception:
      parser_mediator.ProduceExtractionWarning('{0!s}'.format(exception))

    finally:
      if registry_file:
        registry_file.Close()


manager.ParsersManager.RegisterParser(WinRegistryParser)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for fseventsd file parser."""

import unittest

from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.path import factory as path_spec_factory
from dfvfs.resolver import resolver as path_spec_resolver

from plaso.parsers import fseventsd

from tests.parsers import test_lib


class FSEventsdParserTest(test_lib.ParserTestCase):
  """Tests for the fseventsd parser."""

  def testParseV1(self):
    """Tests the Parse function on a version 1 file."""
    parser = fseventsd.FseventsdParser()

    path = self._GetTestFilePath(['fsevents', 'fsevents-0000000002d89b58'])
    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=path)

    gzip_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_GZIP, parent=os_path_spec)

    storage_writer = self._ParseFileByPathSpec(gzip_path_spec, parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 12)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # The date and time are derived from the file entry.
    os_file_entry = path_spec_resolver.Resolver.OpenFileEntry(os_path_spec)
    date_time_string = (
        os_file_entry.modification_time.CopyToDateTimeStringISO8601())

    expected_event_values = {
        'data_type': 'macos:fseventsd:record',
        'event_identifier': 47747061,
        'file_entry_modification_time': date_time_string,
        'flags': 0x01000080,
        'path': '.Spotlight-V100/Store-V1'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 3)
    self.CheckEventData(event_data, expected_event_values)

  def testParseV2(self):
    """Tests the Parse function on a version 2 file."""
    parser = fseventsd.FseventsdParser()

    path = self._GetTestFilePath(['fsevents', 'fsevents-00000000001a0b79'])
    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=path)

    gzip_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_GZIP, parent=os_path_spec)

    storage_writer = self._ParseFileByPathSpec(gzip_path_spec, parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 6)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # The date and time are derived from the file entry.
    os_file_entry = path_spec_resolver.Resolver.OpenFileEntry(os_path_spec)
    date_time_string = (
        os_file_entry.modification_time.CopyToDateTimeStringISO8601())

    expected_event_values = {
        'data_type': 'macos:fseventsd:record',
        'event_identifier': 1706838,
        'file_entry_modification_time': date_time_string,
        'flags': 0x01000008,
        'path': 'Hi, Sierra'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 2)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Interface for JSON-L parser plugins."""

import abc
import json
import os

from dfdatetime import time_elements as dfdatetime_time_elements

from dfvfs.helpers import text_file

from plaso.parsers import plugins


class JSONLPlugin(plugins.BasePlugin):
  """This is an abstract class from which plugins should be based.

  The following are the attributes and methods expected to be overridden by a
  JSON-L parser plugin.
  """

  NAME = 'jsonl_plugin'

  def _GetJSONValue(self, json_dict, name, default_value=None):
    """Retrieves a value from a JSON dict.

    Args:
      json_dict (dict): JSON dictionary.
      name (str): name of the value to retrieve.
      default_value (Optional[object]): default value to return if the value
          is not set or empty.

    Returns:
      object: value of the JSON log entry or None if not set.
    """
    json_value = json_dict.get(name, default_value)
    if json_value == '':
      json_value = default_value
    return json_value

  def _ParseISO8601DateTimeString(self, parser_mediator, json_dict, name):
    """Parses an ISO8601 date and time string.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      json_dict (dict): JSON dictionary.
      name (str): name of the value to retrieve.

    Returns:
      dfdatetime.TimeElementsInMicroseconds: date and time value or None if
          not available.
    """
    iso8601_string = self._GetJSONValue(json_dict, name)
    if not iso8601_string:
      return None

    try:
      date_time = dfdatetime_time_elements.TimeElementsInMicroseconds()
      date_time.CopyFromStringISO8601(iso8601_string)
    except ValueError as exception:
      parser_mediator.ProduceExtractionWarning((
          'Unable to parse value: {0:s} ISO8601 string: {1:s} with error: '
          '{2!s}').format(name, iso8601_string, exception))
      return None

    return date_time

  @abc.abstractmethod
  def _ParseRecord(self, parser_mediator, json_dict):
    """Parses a JSON-L log record structure and produces events.

    This function takes as an input a parsed JSON dictionary and produces
    events.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      json_dict (dict): JSON dictionary of the log record.
    """

  @abc.abstractmethod
  def CheckRequiredFormat(self, json_dict):
    """Check if the log record has the minimal structure required by the plugin.

    Args:
      json_dict (dict): JSON dictionary of the log record.

    Returns:
      bool: True if this is the correct parser, False otherwise.
    """

  # pylint: disable=arguments-differ
  def Process(self, parser_mediator, file_object=None, **kwargs):
    """Extracts events from a JSON-L log file.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      file_object (Optional[dfvfs.FileIO]): a file-like object.
    """
    # This will raise if unhandled keyword arguments are passed.
    super(JSONLPlugin, self).Process(parser_mediator)

    file_object.seek(0, os.SEEK_SET)
    text_file_object = text_file.TextFile(file_object)

    # TODO: add support to handle corrupt lines like text parser.
    for line in text_file_object:
      json_dict = json.loads(line)
      self._ParseRecord(parser_mediator, json_dict)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the System Resource Usage Monitor (SRUM) ESE database file."""

import unittest

from plaso.parsers.esedb_plugins import srum

from tests.parsers.esedb_plugins import test_lib


class SystemResourceUsageMonitorESEDBPluginTest(test_lib.ESEDBPluginTestCase):
  """Tests for the System Resource Usage Monitor (SRUM) ESE database plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = srum.SystemResourceUsageMonitorESEDBPlugin()
    storage_writer = self._ParseESEDBFileWithPlugin(['SRUDB.dat'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 18283)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 2)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # Test an entry with data type windows:srum:application_usage
    expected_event_values = {
        'application': 'Memory Compression',
        'data_type': 'windows:srum:application_usage',
        'identifier': 22167,
        'recorded_time': '2017-11-05T11:32:00.000000+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1878)
    self.CheckEventData(event_data, expected_event_values)

    # Test an entry with data type windows:srum:network_connectivity
    expected_event_values = {
        'application': 1,
        'data_type': 'windows:srum:network_connectivity',
        'identifier': 501,
        'last_connected_time': '2017-11-05T10:30:48.1679714+00:00',
        'recorded_time': '2017-11-05T13:33:00.000000+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex(
        'event_data', 18027)
    self.CheckEventData(event_data, expected_event_values)

    # Test an entry with data type windows:srum:network_usage
    expected_event_values = {
        'application': 'DiagTrack',
        'bytes_sent': 2076,
        'data_type': 'windows:srum:network_usage',
        'identifier': 3495,
        'interface_luid': 1689399632855040,
        'recorded_time': '2017-11-05T11:32:00.000000+00:00',
        'user_identifier': 'S-1-5-18'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""This file contains the tests for the generic text parser."""

import unittest

from dfvfs.file_io import fake_file_io
from dfvfs.path import fake_path_spec
from dfvfs.resolver import context as dfvfs_context

from plaso.parsers import text_parser

from tests.parsers import test_lib


class EncodedTextReaderTest(test_lib.ParserTestCase):
  """Tests for encoded text reader."""

  _TEST_LINES ='\n'.join([
      'Multiple lines',
      'of text',
      'in a single',
      'file.'])

  _TEST_DATA = _TEST_LINES.encode('utf-8')

  def _EncodingErrorHandler(self, exception):
    """Encoding error handler.

    Args:
      exception [UnicodeDecodeError]: exception.

    Returns:
      tuple[str, int]: replacement string and number of bytes to skip.

    Raises:
      TypeError: if exception is not of type UnicodeDecodeError.
    """
    if not isinstance(exception, UnicodeDecodeError):
      raise TypeError('Unsupported exception type.')

    # pylint: disable=attribute-defined-outside-init,no-member
    self._encoding_errors.append(
        (exception.start, exception.object[exception.start]))
    escaped = '\\x{0:2x}'.format(exception.object[exception.start])
    return (escaped, exception.start + 1)

  def testReadLine(self):
    """Tests the ReadLine function."""
    resolver_context = dfvfs_context.Context()

    test_path_spec = fake_path_spec.FakePathSpec(location='/file.txt')
    file_object = fake_file_io.FakeFile(
        resolver_context, test_path_spec, self._TEST_DATA)
    file_object.Open()

    text_reader = text_parser.EncodedTextReader(file_object)

    line = text_reader.ReadLine()
    self.assertEqual(line, 'Multiple lines')

    line = text_reader.ReadLine()
    self.assertEqual(line, 'of text')

  def testReadLines(self):
    """Tests the ReadLines function."""
    resolver_context = dfvfs_context.Context()

    test_path_spec = fake_path_spec.FakePathSpec(location='/file.txt')
    file_object = fake_file_io.FakeFile(
        resolver_context, test_path_spec, self._TEST_DATA)
    file_object.Open()

    text_reader = text_parser.EncodedTextReader(file_object)

    text_reader.ReadLines()
    self.assertEqual(text_reader.lines, self._TEST_LINES)

  def testSkipAhead(self):
    """Tests the SkipAhead function."""
    resolver_context = dfvfs_context.Context()

    test_path_spec = fake_path_spec.FakePathSpec(location='/file.txt')
    file_object = fake_file_io.FakeFile(
        resolver_context, test_path_spec, self._TEST_DATA)
    file_object.Open()

    text_reader = text_parser.EncodedTextReader(file_object)

    text_reader.SkipAhead(10)
    self.assertEqual(text_reader.lines, self._TEST_LINES[10:])


class TextLogParserTest(test_lib.ParserTestCase):
  """Tests for the text log parser."""

  # pylint: disable=protected-access

  def testContainsBinary(self):
    """Tests the _ContainsBinary function."""
    parser = text_parser.TextLogParser()

    result = parser._ContainsBinary('this it text\n')
    self.assertFalse(result)

    result = parser._ContainsBinary('\x01\x00binary\x02\x00')
    self.assertTrue(result)

  def testEnablePlugins(self):
    """Tests the EnablePlugins function."""
    parser = text_parser.TextLogParser()

    number_of_plugins = len(parser._plugin_classes)

    parser.EnablePlugins([])
    self.assertEqual(len(parser._plugins_per_name), 0)

    parser.EnablePlugins(parser.ALL_PLUGINS)
    self.assertEqual(len(parser._plugins_per_name), number_of_plugins)

    parser.EnablePlugins(['apache_access'])
    self.assertEqual(len(parser._plugins_per_name), 1)

  # TODO: add tests for ParseFileObject


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the count related attribute containers."""

import unittest

from plaso.containers import counts

from tests import test_lib as shared_test_lib


class EventLabelCountTest(shared_test_lib.BaseTestCase):
  """Tests for the event label count attribute container."""

  def testGetAttributeNames(self):
    """Tests the GetAttributeNames function."""
    attribute_container = counts.EventLabelCount()

    expected_attribute_names = [
        'label',
        'number_of_events']

    attribute_names = sorted(attribute_container.GetAttributeNames())
    self.assertEqual(attribute_names, expected_attribute_names)


class ParserCountTest(shared_test_lib.BaseTestCase):
  """Tests for the parser count attribute container."""

  def testGetAttributeNames(self):
    """Tests the GetAttributeNames function."""
    attribute_container = counts.ParserCount()

    expected_attribute_names = [
        'name',
        'number_of_events']

    attribute_names = sorted(attribute_container.GetAttributeNames())
    self.assertEqual(attribute_names, expected_attribute_names)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Text parser plugin related functions and classes for testing."""

from plaso.containers import events
from plaso.parsers import mediator as parsers_mediator
from plaso.parsers import text_parser

from tests.parsers import test_lib


class TextPluginTestCase(test_lib.ParserTestCase):
  """Text parser plugin test case."""

  def _ParseTextFileWithPlugin(self, path_segments, plugin):
    """Parses a file as a text log file and returns an event generator.

    This method will first test if a text log file has the required format
    using plugin.CheckRequiredFormat() and then extracts events using
    plugin.Process().

    Args:
      path_segments (list[str]): path segments inside the test data directory.
      plugin (TextPlugin): text log file plugin.

    Returns:
      FakeStorageWriter: storage writer.

    Raises:
      SkipTest: if the path inside the test data directory does not exist and
          the test should be skipped.
    """
    parser_mediator = parsers_mediator.ParserMediator()

    storage_writer = self._CreateStorageWriter()
    parser_mediator.SetStorageWriter(storage_writer)

    file_entry = self._GetTestFileEntry(path_segments)
    parser_mediator.SetFileEntry(file_entry)

    if file_entry:
      event_data_stream = events.EventDataStream()
      event_data_stream.path_spec = file_entry.path_spec

      parser_mediator.ProduceEventDataStream(event_data_stream)

    # AppendToParserChain needs to be run after SetFileEntry.
    parser_mediator.AppendToParserChain('text')

    encoding = plugin.ENCODING
    if not encoding:
      encoding = parser_mediator.GetCodePage()

    file_object = file_entry.GetFileObject()
    text_reader = text_parser.EncodedTextReader(file_object, encoding=encoding)

    text_reader.ReadLines()

    required_format = plugin.CheckRequiredFormat(parser_mediator, text_reader)
    self.assertTrue(required_format)

    plugin.UpdateChainAndProcess(parser_mediator, file_object=file_object)

    if hasattr(plugin, 'GetDateLessLogHelper'):
      date_less_log_helper = plugin.GetDateLessLogHelper()
      parser_mediator.AddDateLessLogHelper(date_less_log_helper)

    return storage_writer



# -*- coding: utf-8 -*-
"""Text parser plugin for Mac OS launchd log files."""

import pyparsing

from dfdatetime import time_elements as dfdatetime_time_elements

from plaso.containers import events
from plaso.lib import errors
from plaso.parsers import text_parser
from plaso.parsers.text_plugins import interface


class MacOSLaunchdEventData(events.EventData):
  """Mac OS launchd log event data.

  Attributes:
    body (str): content of the log event.
    process_name (str): name of the process that created the record.
    severity (str): severity of the message.
    written_time (dfdatetime.DateTimeValues): date and time the log entry was
        written.
  """

  DATA_TYPE = 'macos:launchd_log:entry'

  def __init__(self):
    """Initializes event data."""
    super(MacOSLaunchdEventData, self).__init__(data_type=self.DATA_TYPE)
    self.body = None
    self.process_name = None
    self.severity = None
    self.written_time = None


class MacOSLaunchdLogTextPlugin(interface.TextPlugin):
  """Text parser plugin for Mac OS launchd log files."""

  NAME = 'macos_launchd_log'
  DATA_FORMAT = 'Mac OS launchd log file'

  # Date and time values are formatted as:
  # 2023-06-08 14:51:38.987368
  _DATE_TIME = pyparsing.Regex(
      r'(?P<date_time>[0-9]{4}-[0-9]{2}-[0-9]{2} '
      r'[0-9]{2}:[0-9]{2}:[0-9]{2}.[0-9]{6}) ')

  _PROCESS_NAME = pyparsing.Regex(r'[(](?P<process_name>[^)]+)[)] ')

  _SEVERITY = pyparsing.Regex(r'[<](?P<severity>[^>]+)[>]: ')

  _END_OF_LINE = pyparsing.Suppress(pyparsing.LineEnd())

  _LOG_LINE = (
      _DATE_TIME + pyparsing.Optional(_PROCESS_NAME) + _SEVERITY +
      pyparsing.restOfLine().set_results_name('body') + _END_OF_LINE)

  _LINE_STRUCTURES = [('log_line', _LOG_LINE)]

  VERIFICATION_GRAMMAR = _LOG_LINE

  def _ParseRecord(self, parser_mediator, key, structure):
    """Parses a pyparsing structure.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      key (str): name of the parsed structure.
      structure (pyparsing.ParseResults): tokens from a parsed log line.

    Raises:
      ParseError: if the structure cannot be parsed.
    """
    if key == 'log_line':
      time_elements_structure = self._GetValueFromStructure(
          structure, 'date_time')

      event_data = MacOSLaunchdEventData()
      event_data.body = self._GetValueFromStructure(structure, 'body')
      event_data.process_name = self._GetValueFromStructure(
          structure, 'process_name')
      event_data.severity = self._GetValueFromStructure(structure, 'severity')
      event_data.written_time = self._ParseTimeElements(time_elements_structure)

      parser_mediator.ProduceEventData(event_data)

  def _ParseTimeElements(self, time_elements_structure):
    """Parses date and time elements.

    Args:
      time_elements_structure (pyparsing.ParseResults): date and time elements
          of a log line.

    Returns:
      dfdatetime.TimeElements: date and time value.

    Raises:
      ParseError: if a valid date and time value cannot be derived from
          the time elements.
    """
    try:
      date_time = dfdatetime_time_elements.TimeElementsInMicroseconds()
      date_time.CopyFromDateTimeString(time_elements_structure)

      return date_time

    except (TypeError, ValueError) as exception:
      raise errors.ParseError(
          f'Unable to parse time elements with error: {exception!s}')

  def CheckRequiredFormat(self, parser_mediator, text_reader):
    """Check if the log record has the minimal structure required by the parser.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      text_reader (EncodedTextReader): text reader.

    Returns:
      bool: True if this is the correct plugin, False otherwise.
    """
    try:
      structure = self._VerifyString(text_reader.lines)
    except errors.ParseError:
      return False

    time_elements_structure = self._GetValueFromStructure(
        structure, 'date_time')

    try:
      self._ParseTimeElements(time_elements_structure)
    except errors.ParseError:
      return False

    return True


text_parser.TextLogParser.RegisterPlugin(MacOSLaunchdLogTextPlugin)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the MountPoints2 Windows Registry plugin."""

import unittest

from plaso.parsers.winreg_plugins import mountpoints

from tests.parsers.winreg_plugins import test_lib


class MountPoints2PluginTest(test_lib.RegistryPluginTestCase):
  """Tests for the MountPoints2 Windows Registry plugin."""

  def testFilters(self):
    """Tests the FILTERS class attribute."""
    plugin = mountpoints.MountPoints2Plugin()

    self._AssertFiltersOnKeyPath(plugin, 'HKEY_CURRENT_USER', (
        'Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\MountPoints2'))

    self._AssertNotFiltersOnKeyPath(plugin, 'HKEY_CURRENT_USER', 'Bogus')

  def testProcess(self):
    """Tests the Process function."""
    test_file_entry = self._GetTestFileEntry(['NTUSER-WIN7.DAT'])
    key_path = (
        'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
        'Explorer\\MountPoints2')

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)

    plugin = mountpoints.MountPoints2Plugin()
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 5)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'windows:registry:mount_points2',
        'key_path': key_path,
        'label': 'Home Drive',
        'last_written_time': '2011-08-23T17:10:14.9609605+00:00',
        'name': '##controller#home#nfury',
        'server_name': 'controller',
        'share_name': '\\home\\nfury',
        'type': 'Remote Drive'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the .automaticDestinations-ms OLECF parser plugin."""

import unittest

from plaso.parsers.olecf_plugins import automatic_destinations

from tests.parsers.olecf_plugins import test_lib


class TestAutomaticDestinationsOLECFPlugin(test_lib.OLECFPluginTestCase):
  """Tests for the .automaticDestinations-ms OLECF parser plugin."""

  def testProcessVersion1(self):
    """Tests the Process function on version 1 .automaticDestinations-ms."""
    plugin = automatic_destinations.AutomaticDestinationsOLECFPlugin()
    storage_writer = self._ParseOLECFFileWithPlugin(
        ['automatic_destinations', '1b4dd67f29cb1962.automaticDestinations-ms'],
        plugin)

    # Number of events:
    # olecf:dest_list:entry: 11
    # windows:lnk:link 33
    # windows:distributed_link_tracking:creation: 44

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 55)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # Check a AutomaticDestinationsDestListEntryEvent.
    expected_event_values = {
        'birth_droid_file_identifier': '{63eea867-7b85-11e1-8950-005056a50b40}',
        'birth_droid_volume_identifier': (
            '{cf6619c2-66a8-44a6-8849-1582fcd3a338}'),
        'data_type': 'olecf:dest_list:entry',
        'droid_file_identifier': '{63eea867-7b85-11e1-8950-005056a50b40}',
        'droid_volume_identifier': '{cf6619c2-66a8-44a6-8849-1582fcd3a338}',
        'entry_number': 11,
        'hostname': 'wks-win764bitb',
        'modification_time': '2012-04-01T13:52:38.9975382+00:00',
        'offset': 32,
        'path': 'C:\\Users\\nfury\\Pictures\\The SHIELD',
        'pin_status': -1}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 4)
    self.CheckEventData(event_data, expected_event_values)

    # Check a WinLnkLinkEvent.
    expected_event_values = {
        'access_time': '2010-11-10T07:51:23.1085000+00:00',
        'creation_time': '2010-11-10T07:51:16.7491250+00:00',
        'data_type': 'windows:lnk:link',
        'drive_serial_number': 0x24ba718b,
        'drive_type': 3,
        'file_attribute_flags': 0x00002020,
        'file_size': 3545,
        'link_target': '<Users Libraries> <Users property view>',
        'local_path': (
            'C:\\\\Users\\\\nfury\\\\AppData\\\\Roaming\\\\Microsoft\\\\'
            'Windows\\\\Libraries\\\\Documents.library-ms'),
        'modification_time': '2010-11-10T07:51:23.1085000+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    # Check a WindowsDistributedLinkTrackingCreationEvent.
    expected_event_values = {
        'creation_time': '2012-03-31T23:01:03.5277415+00:00',
        'data_type': 'windows:distributed_link_tracking:creation',
        'mac_address': '00:50:56:a5:0b:40',
        'origin': 'DestList entry at offset: 0x00000020',
        'uuid': '63eea867-7b85-11e1-8950-005056a50b40'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 3)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessVersion3(self):
    """Tests the Process function on version 3 .automaticDestinations-ms."""
    plugin = automatic_destinations.AutomaticDestinationsOLECFPlugin()
    storage_writer = self._ParseOLECFFileWithPlugin(
        ['automatic_destinations', '9d1f905ce5044aee.automaticDestinations-ms'],
        plugin)

    # Event data types:
    # olecf:dest_list:entry: 2
    # windows:lnk:link 2

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 4)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # Check a AutomaticDestinationsDestListEntryEvent.
    expected_event_values = {
        'birth_droid_file_identifier': '{00000000-0000-0000-0000-000000000000}',
        'birth_droid_volume_identifier': (
            '{00000000-0000-0000-0000-000000000000}'),
        'data_type': 'olecf:dest_list:entry',
        'droid_file_identifier': '{00000000-0000-0000-0000-000000000000}',
        'droid_volume_identifier': '{00000000-0000-0000-0000-000000000000}',
        'entry_number': 2,
        'hostname': None,
        'modification_time': '2016-01-17T13:08:08.2475045+00:00',
        'offset': 32,
        'path': 'http://support.microsoft.com/kb/3124263',
        'pin_status': -1}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)

    # Check a WinLnkLinkEvent.
    expected_event_values = {
        'access_time': None,
        'creation_time': None,
        'data_type': 'windows:lnk:link',
        'drive_serial_number': None,
        'drive_type': None,
        'file_attribute_flags': 0,
        'file_size': 0,
        'link_target': '<Internet Folder> <UNKNOWN: 0x61>',
        'local_path': None,
        'modification_time': None}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Shared functionality for text file based output modules."""

import abc
import heapq
import os

from plaso.output import interface


class SortedStringHeap(object):
  """Heap to sort output strings."""

  _MAXIMUM_NUMBER_OF_STRINGS = 100000

  def __init__(self):
    """Initializes a heap."""
    super(SortedStringHeap, self).__init__()
    self._heap = []

  def IsFull(self):
    """Determines if the heap is full.

    Returns:
      bool: True if the heap is full.
    """
    return len(self._heap) >= self._MAXIMUM_NUMBER_OF_STRINGS

  def PopString(self):
    """Pops a string from the heap.

    Returns:
      str: string.
    """
    try:
      _, string = heapq.heappop(self._heap)
    except IndexError:
      return None

    return string

  def PopStrings(self):
    """Pops strings from the heap.

    Yields:
      str: string.
    """
    string = self.PopString()
    while string:
      yield string
      string = self.PopString()

  def PushString(self, sort_key, string):
    """Pushes a string onto the heap.

    Args:
      sort_key (str): key for the sort order.
      string (str): string.
    """
    heapq.heappush(self._heap, (sort_key, string))


class TextFileOutputModule(interface.OutputModule):
  """Shared functionality of an output module that writes to a text file."""

  WRITES_OUTPUT_FILE = True

  _ENCODING = 'utf-8'

  def __init__(self):
    """Initializes an output module that writes to a text file."""
    super(TextFileOutputModule, self).__init__()
    self._file_object = None

  def Close(self):
    """Closes the output file."""
    if self._file_object:
      self._file_object.close()
      self._file_object = None

  @abc.abstractmethod
  def GetFieldValues(
      self, output_mediator, event, event_data, event_data_stream, event_tag):
    """Retrieves the output field values.

    Args:
      output_mediator (OutputMediator): mediates interactions between output
          modules and other components, such as storage and dfVFS.
      event (EventObject): event.
      event_data (EventData): event data.
      event_data_stream (EventDataStream): event data stream.
      event_tag (EventTag): event tag.

    Returns:
      dict[str, str]: output field values per name.
    """

  def Open(self, path=None, **kwargs):  # pylint: disable=arguments-differ
    """Opens the output file.

    Args:
      path (Optional[str]): path of the output file.

    Raises:
      IOError: if the specified output file already exists.
      OSError: if the specified output file already exists.
      ValueError: if path is not set.
    """
    if not path:
      raise ValueError('Missing path.')

    if os.path.isfile(path):
      raise IOError((
          'Unable to use an already existing file for output '
          '[{0:s}]').format(path))

    self._file_object = open(path, 'wt', encoding=self._ENCODING)  # pylint: disable=consider-using-with

  @abc.abstractmethod
  def WriteFieldValues(self, output_mediator, field_values):
    """Writes field values to the output.

    Args:
      output_mediator (OutputMediator): mediates interactions between output
          modules and other components, such as storage and dfVFS.
      field_values (dict[str, str]): output field values per name.
    """

  def WriteLine(self, text):
    """Writes a line of text to the output file.

    Args:
      text (str): text to output.
    """
    self._file_object.write('{0:s}\n'.format(text))

  def WriteText(self, text):
    """Writes text to the output file.

    Args:
      text (str): text to output.
    """
    self._file_object.write(text)


class SortedTextFileOutputModule(TextFileOutputModule):
  """Shared functionality of an output module that writes to a text file."""

  _SORT_KEY_FIELD_NAMES = ['time']

  def __init__(self, event_formatting_helper):
    """Initializes an output module that writes to a text file.

    Args:
      event_formatting_helper (EventFormattingHelper): event formatting helper.
    """
    super(SortedTextFileOutputModule, self).__init__()
    self._event_formatting_helper = event_formatting_helper
    self._last_primary_sort_key = None
    self._sorted_strings_heap = SortedStringHeap()

  def _FlushSortedStringsHeap(self):
    """Flushed the sorted strings heap."""
    for output_text in self._sorted_strings_heap.PopStrings():
      self.WriteText(output_text)

    self._last_primary_sort_key = None

  @abc.abstractmethod
  def _GetString(self, output_mediator, field_values):
    """Retrieves an output string.

    Args:
      output_mediator (OutputMediator): mediates interactions between output
          modules and other components, such as storage and dfVFS.
      field_values (dict[str, str]): output field values per name.

    Returns:
      str: output string.
    """

  def GetFieldValues(
      self, output_mediator, event, event_data, event_data_stream, event_tag):
    """Retrieves the output field values.

    Args:
      output_mediator (OutputMediator): mediates interactions between output
          modules and other components, such as storage and dfVFS.
      event (EventObject): event.
      event_data (EventData): event data.
      event_data_stream (EventDataStream): event data stream.
      event_tag (EventTag): event tag.

    Returns:
      dict[str, str]: output field values per name.
    """
    return self._event_formatting_helper.GetFieldValues(
        output_mediator, event, event_data, event_data_stream, event_tag)

  def WriteFieldValues(self, output_mediator, field_values):
    """Writes field values to the output.

    Args:
      output_mediator (OutputMediator): mediates interactions between output
          modules and other components, such as storage and dfVFS.
      field_values (dict[str, str]): output field values per name.
    """
    primary_sort_key = field_values.get(self._SORT_KEY_FIELD_NAMES[0], None)
    if self._last_primary_sort_key is None:
      self._last_primary_sort_key = primary_sort_key

    if (primary_sort_key != self._last_primary_sort_key or
        self._sorted_strings_heap.IsFull()):
      self._FlushSortedStringsHeap()

    output_text = self._GetString(output_mediator, field_values)
    if output_text:
      sort_key = ' '.join([field_values.get(field_name, None) or ''
                           for field_name in self._SORT_KEY_FIELD_NAMES])
      self._sorted_strings_heap.PushString(sort_key, output_text)

  def WriteFooter(self):
    """Writes the footer to the output.

    Can be used for post-processing or output after the last event is written,
    such as writing a file footer.
    """
    self._FlushSortedStringsHeap()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the File History ESE database file."""

import unittest

from plaso.parsers.esedb_plugins import file_history

from tests.parsers.esedb_plugins import test_lib


class FileHistoryESEDBPluginTest(test_lib.ESEDBPluginTestCase):
  """Tests for the File History ESE database plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = file_history.FileHistoryESEDBPlugin()
    storage_writer = self._ParseESEDBFileWithPlugin(['Catalog1.edb'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1373)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'creation_time': '2013-09-02T20:02:25.6957745+00:00',
        'data_type': 'windows:file_history:namespace',
        'file_attribute': 16,
        'identifier': 356,
        'modification_time': '2013-10-12T17:34:36.6885806+00:00',
        'original_filename': '?UP\\Favorites\\Links\\Lenovo',
        'parent_identifier': 230,
        'usn_number': 9251162904}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 355)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Apple account plist plugin."""

import unittest

from plaso.parsers.plist_plugins import apple_account

from tests.parsers.plist_plugins import test_lib


class AppleAccountPlistPluginTest(test_lib.PlistPluginTestCase):
  """Tests for the Apple account plist plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plist_name = (
        'com.apple.coreservices.appleidauthenticationinfo.'
        'ABC0ABC1-ABC0-ABC0-ABC0-ABC0ABC1ABC2.plist')

    plugin = apple_account.AppleAccountPlistPlugin()
    storage_writer = self._ParsePlistFileWithPlugin(
        plugin, [plist_name], plist_name)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'account_name': 'email@domain.com',
        'data_type': 'macos:apple_account:entry',
        'creation_time': '2013-06-24T20:46:42.000000+00:00',
        'first_name': 'Joaquin',
        'last_connected_time': '2013-12-25T14:00:32.000000+00:00',
        'last_name': 'Moreno Garijo',
        'validation_time': '2013-12-25T14:00:32.000000+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*_ coding: utf-8 -*-
"""Text parser plugin for System Center Configuration Manager (SCCM) logs."""

import re

from dfdatetime import time_elements as dfdatetime_time_elements

import pyparsing

from plaso.containers import events
from plaso.lib import errors
from plaso.parsers import text_parser
from plaso.parsers.text_plugins import interface


class SCCMLogEventData(events.EventData):
  """SCCM log event data.

  Attributes:
    component (str): component.
    text (str): text.
    written_time (dfdatetime.DateTimeValues): date and time the entry was
        written.
  """

  DATA_TYPE = 'sccm_log:entry'

  def __init__(self):
    """Initializes event data."""
    super(SCCMLogEventData, self).__init__(data_type=self.DATA_TYPE)
    self.component = None
    self.text = None
    self.written_time = None


class SCCMTextPlugin(interface.TextPlugin):
  """Text parser plugin for System Center Configuration Manager (SCCM) logs."""

  NAME = 'sccm'
  DATA_FORMAT = 'System Center Configuration Manager (SCCM) client log file'

  ENCODING = 'utf-8'

  _ONE_OR_TWO_DIGITS = pyparsing.Word(pyparsing.nums, max=2).set_parse_action(
      lambda tokens: int(tokens[0], 10))

  _TWO_OR_THREE_DIGITS = pyparsing.Word(
      pyparsing.nums, min=2, max=3).set_parse_action(
          lambda tokens: int(tokens[0], 10))

  _TWO_DIGITS = pyparsing.Word(pyparsing.nums, exact=2).set_parse_action(
      lambda tokens: int(tokens[0], 10))

  _FOUR_DIGITS = pyparsing.Word(pyparsing.nums, exact=4).set_parse_action(
      lambda tokens: int(tokens[0], 10))

  # Date formatted as: date="M-D-YYYY"
  _DATE = (pyparsing.Suppress('" date="') + _ONE_OR_TWO_DIGITS +
           pyparsing.Suppress('-') + _ONE_OR_TWO_DIGITS +
           pyparsing.Suppress('-') + _FOUR_DIGITS)

  _TIME_ZONE_OFFSET = pyparsing.Group(
      pyparsing.Word('+-', exact=1) + _TWO_OR_THREE_DIGITS)

  # Time formatted as: time="h:mm:ss.###or time="h:mm:ss.###[+-]##"
  _TIME = (pyparsing.Suppress('time="') + _ONE_OR_TWO_DIGITS +
           pyparsing.Suppress(':') + _TWO_DIGITS + pyparsing.Suppress(':') +
           _TWO_DIGITS + pyparsing.Suppress('.') + pyparsing.Regex(r'\d{3,7}') +
           pyparsing.Optional(_TIME_ZONE_OFFSET))

  _DATE_TIME = (_TIME + _DATE).set_results_name('date_time')

  _LOG_MESSAGE_START = pyparsing.Suppress('<![LOG[')

  # Using a regular expression look ahead here is faster than pyparsing.SkipTo()
  _LOG_MESSAGE_TEXT = pyparsing.Regex(r'.*?(?=(]LOG]!><))', re.DOTALL)

  _LOG_MESSAGE_END = pyparsing.Suppress(']LOG]!><')

  _LOG_MESSAGE = (
      _LOG_MESSAGE_START + _LOG_MESSAGE_TEXT.set_results_name('text') +
      _LOG_MESSAGE_END)

  _COMPONENT = (
      pyparsing.Suppress('" component="') +
      pyparsing.Word(pyparsing.alphanums).set_results_name('component'))

  _END_OF_LINE = pyparsing.Suppress(pyparsing.LineEnd())

  _LOG_LINE = (
      _LOG_MESSAGE + _DATE_TIME + _COMPONENT +
      pyparsing.Regex(r'.*?(?=(<!\[LOG\[))', re.DOTALL))

  _LAST_LOG_LINE = (
      _LOG_MESSAGE + _DATE_TIME + _COMPONENT +
      pyparsing.restOfLine() + _END_OF_LINE)

  _LINE_STRUCTURES = [
      ('log_line', _LOG_LINE),
      ('last_log_line', _LAST_LOG_LINE)]

  # Because logs files can lead with a partial event, we can't assume that
  # the first character (post-BOM) in the file is the beginning of our match,
  # so we look for match anywhere in lines.

  VERIFICATION_GRAMMAR = (
      pyparsing.Regex(r'.*<!\[LOG\[.*]LOG]!><', re.DOTALL) + _DATE_TIME +
      _COMPONENT)

  VERIFICATION_LITERALS = ['<![LOG[', ']LOG]!><time="']

  def _ParseRecord(self, parser_mediator, key, structure):
    """Parses a pyparsing structure.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      key (str): name of the parsed structure.
      structure (pyparsing.ParseResults): tokens from a parsed log line.

    Raises:
      ParseError: if the structure cannot be parsed.
    """
    time_elements_structure = self._GetValueFromStructure(
         structure, 'date_time')

    event_data = SCCMLogEventData()
    event_data.component = self._GetValueFromStructure(structure, 'component')
    event_data.text = self._GetValueFromStructure(structure, 'text')
    event_data.written_time = self._ParseTimeElements(time_elements_structure)

    parser_mediator.ProduceEventData(event_data)

  def _ParseTimeElements(self, time_elements_structure):
    """Parses date and time elements of a log line.

    Args:
      time_elements_structure (pyparsing.ParseResults): date and time elements
          of a log line.

    Returns:
      dfdatetime.TimeElements: date and time value.

    Raises:
      ParseError: if a valid date and time value cannot be derived from
          the time elements.
    """
    # Ensure time_elements_tuple is not a pyparsing.ParseResults otherwise
    # copy.deepcopy() of the dfDateTime object will fail on Python 3.8 with:
    # "TypeError: 'str' object is not callable" due to pyparsing.ParseResults
    # overriding __getattr__ with a function that returns an empty string when
    # named token does not exist.

    try:
      if len(time_elements_structure) == 8:
        (hours, minutes, seconds, fraction_of_second, utc_offset_minutes, month,
         day_of_month, year) = time_elements_structure

        time_zone_offset = utc_offset_minutes[1]
        if utc_offset_minutes[0] == '-':
          time_zone_offset *= -1

      else:
        (hours, minutes, seconds, fraction_of_second, month, day_of_month,
         year) = time_elements_structure

        time_zone_offset = None

      if len(fraction_of_second) == 3:
        milliseconds = int(fraction_of_second, 10)
        time_elements_tuple=(
            year, month, day_of_month, hours, minutes, seconds, milliseconds)
        date_time = dfdatetime_time_elements.TimeElementsInMilliseconds(
            time_elements_tuple=time_elements_tuple,
            time_zone_offset=time_zone_offset)

      else:
        # TODO: improve precision support, but for now ignore the 100ns
        # precision.
        microseconds = int(fraction_of_second[:6], 10)
        time_elements_tuple=(
            year, month, day_of_month, hours, minutes, seconds, microseconds)
        date_time = dfdatetime_time_elements.TimeElementsInMicroseconds(
            time_elements_tuple=time_elements_tuple,
            time_zone_offset=time_zone_offset)

      return date_time

    except (TypeError, ValueError) as exception:
      raise errors.ParseError(
          'Unable to parse time elements with error: {0!s}'.format(exception))

  def CheckRequiredFormat(self, parser_mediator, text_reader):
    """Check if the log record has the minimal structure required by the parser.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      text_reader (EncodedTextReader): text reader.

    Returns:
      bool: True if this is the correct plugin, False otherwise.
    """
    try:
      structure = self._VerifyString(text_reader.lines)
    except errors.ParseError:
      return False

    time_elements_structure = self._GetValueFromStructure(
        structure, 'date_time')

    try:
      self._ParseTimeElements(time_elements_structure)
    except errors.ParseError:
      return False

    return True


text_parser.TextLogParser.RegisterPlugin(SCCMTextPlugin)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the output mediator object."""

import unittest

from plaso.containers import artifacts
from plaso.lib import definitions
from plaso.storage.fake import writer as fake_writer
from plaso.output import mediator

from tests.containers import test_lib as containers_test_lib
from tests.output import test_lib


class OutputMediatorTest(test_lib.OutputModuleTestCase):
  """Tests for the output mediator object."""

  # pylint: disable=protected-access

  _TEST_EVENTS = [
      {'data_type': 'test:event',
       'filename': 'log/syslog.1',
       'hostname': 'ubuntu',
       'text': (
           'Reporter <CRON> PID: 8442 (pam_unix(cron:session): session\n '
           'closed for user root)'),
       'timestamp': '2012-06-27 18:17:01',
       'timestamp_desc': definitions.TIME_DESCRIPTION_METADATA_MODIFICATION,
       'username': 'root'}]

  # TODO: add tests for _GetWinevtRcDatabaseReader

  def testReadMessageFormattersFile(self):
    """Tests the _ReadMessageFormattersFile function."""
    test_file_path = self._GetTestFilePath(['formatters', 'format_test.yaml'])
    self._SkipIfPathNotExists(test_file_path)

    storage_writer = fake_writer.FakeStorageWriter()
    storage_writer.Open()

    try:
      output_mediator = mediator.OutputMediator(storage_writer)

      output_mediator._ReadMessageFormattersFile(test_file_path)
      self.assertEqual(len(output_mediator._message_formatters), 2)
    finally:
      storage_writer.Close()

  # TODO: add tests for _ReadSourceMappings

  # TODO: add tests for GetDisplayNameForPathSpec

  def testGetHostname(self):
    """Tests the GetHostname function."""
    system_configuration = artifacts.SystemConfigurationArtifact()
    system_configuration.hostname = artifacts.HostnameArtifact(name='myhost')

    storage_writer = fake_writer.FakeStorageWriter()
    storage_writer.Open()

    try:
      storage_writer.AddAttributeContainer(system_configuration)

      output_mediator = mediator.OutputMediator(storage_writer)

      _, event_data, _ = containers_test_lib.CreateEventFromValues(
          self._TEST_EVENTS[0])

      hostname = output_mediator.GetHostname(event_data)
      self.assertEqual(hostname, 'ubuntu')

      event_data.hostname = None

      hostname = output_mediator.GetHostname(event_data)
      self.assertEqual(hostname, 'myhost')

    finally:
      storage_writer.Close()

  def testGetMACBRepresentation(self):
    """Tests the GetMACBRepresentation function."""
    storage_writer = fake_writer.FakeStorageWriter()
    storage_writer.Open()

    try:
      output_mediator = mediator.OutputMediator(storage_writer)

      event, event_data, _ = containers_test_lib.CreateEventFromValues(
          self._TEST_EVENTS[0])
      macb_representation = output_mediator.GetMACBRepresentation(
          event, event_data)
      self.assertEqual(macb_representation, '..C.')

    finally:
      storage_writer.Close()

  # TODO: add tests for GetMACBRepresentationFromDescriptions
  # TODO: add tests for GetMessageFormatter
  # TODO: add tests for GetRelativePathForPathSpec

  def testGetUsername(self):
    """Tests the GetUsername function."""
    test_user1 = artifacts.UserAccountArtifact(
        identifier='1000', path_separator='\\',
        user_directory='C:\\Users\\testuser1',
        username='testuser1')

    storage_writer = fake_writer.FakeStorageWriter()
    storage_writer.Open()

    try:
      output_mediator = mediator.OutputMediator(storage_writer)

      storage_writer.AddAttributeContainer(test_user1)

      _, event_data, _ = containers_test_lib.CreateEventFromValues(
          self._TEST_EVENTS[0])

      username = output_mediator.GetUsername(event_data)
      self.assertEqual(username, 'root')

      event_data.username = None
      setattr(event_data, 'user_sid', '1000')

      username = output_mediator.GetUsername(event_data)
      self.assertEqual(username, 'testuser1')

      setattr(event_data, 'user_sid', '1001')

      username = output_mediator.GetUsername(event_data)
      self.assertEqual(username, '-')

    finally:
      storage_writer.Close()

  # TODO: add tests for GetWindowsEventMessage

  def testReadMessageFormattersFromDirectory(self):
    """Tests the ReadMessageFormattersFromDirectory function."""
    test_directory_path = self._GetTestFilePath(['formatters'])
    self._SkipIfPathNotExists(test_directory_path)

    storage_writer = fake_writer.FakeStorageWriter()
    storage_writer.Open()

    try:
      output_mediator = mediator.OutputMediator(storage_writer)

      output_mediator.ReadMessageFormattersFromDirectory(test_directory_path)
      self.assertEqual(len(output_mediator._message_formatters), 2)

    finally:
      storage_writer.Close()

  def testReadMessageFormattersFromFile(self):
    """Tests the ReadMessageFormattersFromFile function."""
    test_file_path = self._GetTestFilePath(['formatters', 'format_test.yaml'])
    self._SkipIfPathNotExists(test_file_path)

    storage_writer = fake_writer.FakeStorageWriter()
    storage_writer.Open()

    try:
      output_mediator = mediator.OutputMediator(storage_writer)

      output_mediator.ReadMessageFormattersFromFile(test_file_path)
      self.assertEqual(len(output_mediator._message_formatters), 2)

      with self.assertRaises(KeyError):
        output_mediator.ReadMessageFormattersFromFile(test_file_path)

      output_mediator.ReadMessageFormattersFromFile(
          test_file_path, override_existing=True)
      self.assertEqual(len(output_mediator._message_formatters), 2)

    finally:
      storage_writer.Close()

  # TODO: add tests for SetPreferredLanguageIdentifier
  # TODO: add tests for SetTimezone


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""SQLite parser plugin for iOS powerlog database files."""

from dfdatetime import posix_time as dfdatetime_posix_time

from plaso.containers import events
from plaso.parsers import sqlite
from plaso.parsers.sqlite_plugins import interface


class IOSPowerlogApplicationUsageEventData(events.EventData):
  """iOS powerlog file application usage event data.

  Attributes:
    background_time (str): Number of seconds that the application ran in the
        background.
    bundle_identifier (str): Name of the application.
    screen_on_time (str): Number of seconds that the application ran in the
        foreground.
    start_time (dfdatetime.DateTimeValues): date and time the start of
        the application.
  """

  DATA_TYPE = 'ios:powerlog:application_usage'

  def __init__(self):
    """Initializes event data."""
    super(IOSPowerlogApplicationUsageEventData, self).__init__(
        data_type=self.DATA_TYPE)
    self.background_time = None
    self.bundle_identifier = None
    self.screen_on_time = None
    self.start_time = None


class IOSPowerlogApplicationUsagePlugin(interface.SQLitePlugin):
  """SQLite parser plugin for iOS powerlog database files."""

  NAME = 'ios_powerlog'
  DATA_FORMAT = 'iOS powerlog SQLite database (CurrentPowerlog.PLSQL) file'

  REQUIRED_STRUCTURE = {
      'PLAppTimeService_Aggregate_AppRunTime': frozenset([
          'timestamp', 'BackgroundTime', 'BundleID', 'ScreenOnTime'])}

  QUERIES = [(
      'SELECT timestamp, BackgroundTime, ScreenOnTime, BundleID FROM '
      'PLAppTimeService_Aggregate_AppRunTime', 'ParseApplicationRunTime')]

  SCHEMAS = {
      'PLAppTimeService_Aggregate_AppRunTime': (
          'CREATE TABLE PLAppTimeService_Aggregate_AppRunTime (id INTEGER '
          'PRIMARY KEY AUTOINCREMENT, timestamp REAL, timeInterval REAL, '
          'BackgroundAudioNowPlayingPluggedInTime REAL, '
          'BackgroundAudioNowPlayingTime REAL, BackgroundAudioPlayingTime REAL,'
          ' BackgroundAudioPlayingTimePluggedIn REAL, '
          'BackgroundLocationAudioPluggedInTime REAL, '
          'BackgroundLocationAudioTime REAL, '
          'BackgroundLocationPluggedInTime REAL, BackgroundLocationTime REAL, '
          'BackgroundPluggedInTime REAL, BackgroundTime REAL, BundleID TEXT, '
          'ScreenOnPluggedInTime REAL, ScreenOnTime REAL)')}

  REQUIRES_SCHEMA_MATCH = False

  def _GetDateTimeRowValue(self, query_hash, row, value_name):
    """Retrieves a date and time value from the row.

    Args:
      query_hash (int): hash of the query, that uniquely identifies the query
          that produced the row.
      row (sqlite3.Row): row.
      value_name (str): name of the value.

    Returns:
      dfdatetime.PosixTime: date and time value or None if not available.
    """
    timestamp = self._GetRowValue(query_hash, row, value_name)
    if timestamp is None:
      return None

    return dfdatetime_posix_time.PosixTime(timestamp=timestamp)

  # pylint: disable=unused-argument
  def ParseApplicationRunTime(
      self, parser_mediator, query, row, **unused_kwargs):
    """Parses an Application Run Time row.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      query (str): query that created the row.
      row (sqlite3.Row): row.
    """
    query_hash = hash(query)

    event_data = IOSPowerlogApplicationUsageEventData()
    event_data.background_time = self._GetRowValue(
        query_hash, row, 'BackgroundTime')
    event_data.bundle_identifier = self._GetRowValue(
        query_hash, row, 'BundleID')
    event_data.screen_on_time = self._GetRowValue(
        query_hash, row, 'ScreenOnTime')
    event_data.start_time = self._GetDateTimeRowValue(
        query_hash, row, 'timestamp')

    parser_mediator.ProduceEventData(event_data)


sqlite.SQLiteParser.RegisterPlugin(IOSPowerlogApplicationUsagePlugin)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Test for the null output module."""

import os
import unittest

from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.path import factory as path_spec_factory

from plaso.containers import events
from plaso.lib import definitions
from plaso.output import null

from tests.containers import test_lib as containers_test_lib
from tests.output import test_lib


class NullOutputModuleTest(test_lib.OutputModuleTestCase):
  """Test the null output module."""

  # pylint: disable=protected-access

  _OS_PATH_SPEC = path_spec_factory.Factory.NewPathSpec(
      dfvfs_definitions.TYPE_INDICATOR_OS, location='{0:s}{1:s}'.format(
          os.path.sep, os.path.join('cases', 'image.dd')))

  _TEST_EVENTS = [
      {'data_type': 'test:output',
       'display_name': 'OS: /var/log/syslog.1',
       'hostname': 'ubuntu',
       'inode': 12345678,
       'path_spec': path_spec_factory.Factory.NewPathSpec(
           dfvfs_definitions.TYPE_INDICATOR_TSK, inode=15,
           location='/var/log/syslog.1', parent=_OS_PATH_SPEC),
       'text': (
           'Reporter <CRON> PID: |8442| (pam_unix(cron:session): session\n '
           'closed for user root)'),
       'timestamp': '2012-06-27 18:17:01',
       'timestamp_desc': definitions.TIME_DESCRIPTION_UNKNOWN,
       'username': 'root'}]

  def testGetFieldValues(self):
    """Tests the GetFieldValues function."""
    output_mediator = self._CreateOutputMediator()

    output_module = null.NullOutputModule()

    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[0]))

    event_tag = events.EventTag()
    event_tag.AddLabel('Test')

    field_values = output_module.GetFieldValues(
        output_mediator, event, event_data, event_data_stream, event_tag)

    self.assertEqual(field_values, {})

  def testWriteFieldValues(self):
    """Tests the WriteFieldValues function."""
    output_mediator = self._CreateOutputMediator()

    output_module = null.NullOutputModule()

    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[0]))

    event_tag = events.EventTag()
    event_tag.AddLabel('Test')

    field_values = output_module.GetFieldValues(
        output_mediator, event, event_data, event_data_stream, event_tag)

    output_module.WriteFieldValues(output_mediator, field_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the WinRAR Windows Registry plugin."""

import unittest

from dfdatetime import filetime as dfdatetime_filetime
from dfwinreg import definitions as dfwinreg_definitions
from dfwinreg import fake as dfwinreg_fake

from plaso.parsers.winreg_plugins import winrar

from tests.parsers.winreg_plugins import test_lib


class WinRARHistoryPluginTest(test_lib.RegistryPluginTestCase):
  """Tests for the WinRAR history Windows Registry plugin."""

  def _CreateTestKey(self):
    """Creates WinRAR history Registry keys and values for testing.

    Returns:
      dfwinreg.WinRegistryKey: a Windows Registry key.
    """
    filetime = dfdatetime_filetime.Filetime()
    filetime.CopyFromDateTimeString('2012-08-28 09:23:49.002031')
    registry_key = dfwinreg_fake.FakeWinRegistryKey(
        'ArcHistory', key_path_prefix='HKEY_CURRENT_USER',
        last_written_time=filetime.timestamp, offset=1456,
        relative_key_path='Software\\WinRAR\\ArcHistory')

    value_data = 'C:\\Downloads\\The Sleeping Dragon CD1.iso'.encode(
        'utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        '0', data=value_data, data_type=dfwinreg_definitions.REG_SZ,
        offset=1892)
    registry_key.AddValue(registry_value)

    value_data = 'C:\\Downloads\\plaso-static.rar'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        '1', data=value_data, data_type=dfwinreg_definitions.REG_SZ,
        offset=612)
    registry_key.AddValue(registry_value)

    return registry_key

  def testFilters(self):
    """Tests the FILTERS class attribute."""
    plugin = winrar.WinRARHistoryPlugin()

    self._AssertFiltersOnKeyPath(
        plugin, 'HKEY_CURRENT_USER', 'Software\\WinRAR\\ArcHistory')

    self._AssertNotFiltersOnKeyPath(plugin, 'HKEY_LOCAL_MACHINE', 'Bogus')

  def testProcess(self):
    """Tests the Process function."""
    registry_key = self._CreateTestKey()

    plugin = winrar.WinRARHistoryPlugin()
    storage_writer = self._ParseKeyWithPlugin(registry_key, plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'winrar:history',
        'entries': (
            '0: C:\\Downloads\\The Sleeping Dragon CD1.iso '
            '1: C:\\Downloads\\plaso-static.rar'),
        'key_path': 'HKEY_CURRENT_USER\\Software\\WinRAR\\ArcHistory',
        'last_written_time': '2012-08-28T09:23:49.0020310+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Mac OS launchd log text parser plugin."""

import unittest

from plaso.parsers.text_plugins import macos_launchd

from tests.parsers.text_plugins import test_lib


class MacOSLaunchdLogTextPluginTest(test_lib.TextPluginTestCase):
  """Tests for the Mac OS launchd log text parser plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = macos_launchd.MacOSLaunchdLogTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['macos_launchd.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 36609)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'body': (
            'launchd logging initialized. name: com.apple.xpc.launchd pid: 1'),
        'severity': 'Notice',
        'written_time': '2023-06-08T14:51:39.068413+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 12)
    self.CheckEventData(event_data, expected_event_values)

    expected_event_values = {
        'body': 'internal event: PETRIFIED, code = 0',
        'process_name': 'pid/1660/com.apple.audio.AUHostingService.arm64e',
        'severity': 'Notice',
        'written_time': '2023-06-08T11:20:20.231350+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex(
        'event_data', 36603)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Text parser plugin for SELinux audit log (audit.log) files.

audit.log log line example:

type=AVC msg=audit(1105758604.519:420): avc: denied { getattr } for pid=5962
comm="httpd" path="/home/auser/public_html" dev=sdb2 ino=921135

Where msg=audit(1105758604.519:420) contains the number of seconds since
January 1, 1970 00:00:00 UTC and the number of milliseconds after the dot
for example: "seconds: 1105758604, milliseconds: 519".

The number after the timestamp (420 in the example) is a 'serial number'
that can be used to correlate multiple logs generated from the same event.
"""

import pyparsing

from dfdatetime import posix_time as dfdatetime_posix_time

from plaso.containers import events
from plaso.lib import definitions
from plaso.lib import errors
from plaso.parsers import text_parser
from plaso.parsers.text_plugins import interface


class SELinuxLogEventData(events.EventData):
  """SELinux log event data.

  Attributes:
    audit_type (str): audit type.
    body (str): body of the log line.
    last_written_time (dfdatetime.DateTimeValues): entry last written date and
        time.
    pid (int): process identifier (PID) that created the SELinux log line.
  """

  DATA_TYPE = 'selinux:line'

  def __init__(self):
    """Initializes event data."""
    super(SELinuxLogEventData, self).__init__(data_type=self.DATA_TYPE)
    self.audit_type = None
    self.body = None
    self.last_written_time = None
    self.pid = None


class SELinuxTextPlugin(interface.TextPlugin):
  """Text parser plugin for SELinux audit log (audit.log) files."""

  NAME = 'selinux'
  DATA_FORMAT = 'SELinux audit log (audit.log) file'

  _INTEGER = pyparsing.Word(pyparsing.nums).set_parse_action(
      lambda tokens: int(tokens[0], 10))

  _KEY_VALUE_GROUP = pyparsing.Group(
      pyparsing.Word(pyparsing.alphanums) +
      pyparsing.Suppress('=') + (
          pyparsing.QuotedString('"') ^
          pyparsing.Word(pyparsing.printables)))

  _KEY_VALUE_DICT = pyparsing.Dict(
      pyparsing.ZeroOrMore(_KEY_VALUE_GROUP))

  _TIMESTAMP = pyparsing.Group(
      _INTEGER + pyparsing.Suppress('.') + _INTEGER)

  _END_OF_LINE = pyparsing.Suppress(pyparsing.LineEnd())

  # A log line is formatted as: type=TYPE msg=audit([0-9]+\.[0-9]+:[0-9]+): .*
  _LOG_LINE = (
      pyparsing.Suppress('type=') + (
          pyparsing.Word(pyparsing.srange('[A-Z_]')) ^
          pyparsing.Regex(r'UNKNOWN\[[0-9]+\]')).set_results_name('type') +
      pyparsing.Suppress('msg=audit(') +
      _TIMESTAMP.set_results_name('timestamp') +
      pyparsing.Suppress(':') + _INTEGER + pyparsing.Suppress('):') +
      pyparsing.restOfLine().set_results_name('body') +
      _END_OF_LINE)

  _LINE_STRUCTURES = [('log_line', _LOG_LINE)]

  VERIFICATION_GRAMMAR = _LOG_LINE

  def _ParseRecord(self, parser_mediator, key, structure):
    """Parses a pyparsing structure.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      key (str): name of the parsed structure.
      structure (pyparsing.ParseResults): tokens from a parsed log line.

    Raises:
      ParseError: if the structure cannot be parsed.
    """
    if key == 'log_line':
      time_elements_structure = self._GetValueFromStructure(
          structure, 'timestamp')

      # Try to parse the body text as key value pairs. Note that not
      # all log lines will be properly formatted key value pairs.
      body = self._GetValueFromStructure(structure, 'body', default_value='')
      body = body.strip()

      try:
        body_structure = self._KEY_VALUE_DICT.parse_string(body)

        process_identifier = self._GetValueFromStructure(
            body_structure, 'pid')
      except pyparsing.ParseException:
        process_identifier = None

      event_data = SELinuxLogEventData()
      event_data.audit_type = self._GetValueFromStructure(structure, 'type')
      event_data.body = body or None
      event_data.last_written_time = self._ParseTimeElements(
          time_elements_structure)
      event_data.pid = process_identifier

      parser_mediator.ProduceEventData(event_data)

  def _ParseTimeElements(self, time_elements_structure):
    """Parses date and time elements of a log line.

    Args:
      time_elements_structure (pyparsing.ParseResults): date and time elements
          of a log line.

    Returns:
      dfdatetime.PosixTimeInMilliseconds: date and time value.

    Raises:
      ParseError: if a valid date and time value cannot be derived from
          the time elements.
    """
    try:
      seconds, milliseconds = time_elements_structure

      timestamp = (seconds * definitions.MILLISECONDS_PER_SECOND) + milliseconds

      return dfdatetime_posix_time.PosixTimeInMilliseconds(timestamp=timestamp)

    except (TypeError, ValueError) as exception:
      raise errors.ParseError(
          'Unable to parse time elements with error: {0!s}'.format(exception))

  def CheckRequiredFormat(self, parser_mediator, text_reader):
    """Check if the log record has the minimal structure required by the plugin.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      text_reader (EncodedTextReader): text reader.

    Returns:
      bool: True if this is the correct plugin, False otherwise.
    """
    try:
      structure = self._VerifyString(text_reader.lines)
    except errors.ParseError:
      return False

    time_elements_structure = self._GetValueFromStructure(
        structure, 'timestamp')

    try:
      self._ParseTimeElements(time_elements_structure)
    except errors.ParseError:
      return False

    return True


text_parser.TextLogParser.RegisterPlugin(SELinuxTextPlugin)
